@@ -1,9 +1,10 @@
 /*
- *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2021 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *  Author: Simon Désaulniers <simon.desaulniers@gmail.com>
  *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
+ *  Author: Sébastien Blin <sebastien.blin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,16 +31,23 @@
 #include "security/diffie-hellman.h"
 #include "sip/sipaccountbase.h"
 #include "dring/datatransfer_interface.h"
+#include "jamidht/conversation.h"
+#include "multiplexed_socket.h"
 
 #include "noncopyable.h"
 #include "ip_utils.h"
 #include "ring_types.h" // enable_if_base_of
 #include "security/certstore.h"
 #include "scheduled_executor.h"
+#include "connectionmanager.h"
+#include "gitserver.h"
+#include "conversationrepository.h"
 
 #include <opendht/dhtrunner.h>
 #include <opendht/default_types.h>
 
+#include "upnp/protocol/mapping.h"
+
 #include <pjsip/sip_types.h>
 
 #include <vector>
@@ -47,6 +55,7 @@
 #include <chrono>
 #include <list>
 #include <future>
+#include <json/json.h>
 
 #if HAVE_RINGNS
 #include "namedirectory.h"
@@ -70,16 +79,19 @@ class IceTransport;
 struct Contact;
 struct DeviceSync;
 struct AccountArchive;
-class ConnectionManager;
 class DhtPeerConnector;
-class PeerConnection;
 class ContactList;
 class AccountManager;
 struct AccountInfo;
-class ChannelSocket;
 class SipTransport;
 class ChanneledOutgoingTransfer;
 
+using SipConnectionKey = std::pair<std::string /* accountId */, DeviceId>;
+using GitSocketList = std::map<std::string,                            /* device Id */
+                               std::map<std::string,                   /* conversation */
+                                        std::shared_ptr<ChannelSocket> /* related socket */
+                                        >>;
+
 /**
  * @brief Ring Account is build on top of SIPAccountBase and uses DHT to handle call connectivity.
  */
@@ -89,7 +101,7 @@ public:
     constexpr static const char* const ACCOUNT_TYPE = "RING";
     constexpr static const in_port_t DHT_DEFAULT_PORT = 4222;
     constexpr static const char* const DHT_DEFAULT_BOOTSTRAP = "bootstrap.jami.net";
-    constexpr static const char* const DHT_DEFAULT_PROXY = "dhtproxy.jami.net:[80-100]";
+    constexpr static const char* const DHT_DEFAULT_PROXY = "dhtproxy.jami.net:[80-95]";
     constexpr static const char* const DHT_DEFAULT_BOOTSTRAP_LIST_URL
         = "https://config.jami.net/boostrapList";
     constexpr static const char* const DHT_DEFAULT_PROXY_LIST_URL
@@ -124,7 +136,7 @@ public:
      */
     JamiAccount(const std::string& accountID, bool presenceEnabled);
 
-    ~JamiAccount();
+    ~JamiAccount() noexcept;
 
     /**
      * Serialize internal state of this account for configuration
@@ -184,6 +196,14 @@ public:
      */
     void doUnregister(std::function<void(bool)> cb = {}) override;
 
+    /**
+     * Set the registration state of the specified link
+     * @param state The registration state of underlying VoIPLink
+     */
+    void setRegistrationState(RegistrationState state,
+                              unsigned detail_code = 0,
+                              const std::string& detail_str = {}) override;
+
     /**
      * @return pj_str_t "From" uri based on account information.
      * From RFC3261: "The To header field first and foremost specifies the desired
@@ -221,27 +241,8 @@ public:
      */
     pj_str_t getContactHeader(pjsip_transport* = nullptr) override;
 
-    void setReceivedParameter(const std::string& received)
-    {
-        receivedParameter_ = received;
-        via_addr_.host.ptr = (char*) receivedParameter_.c_str();
-        via_addr_.host.slen = receivedParameter_.size();
-    }
-
-    std::string getReceivedParameter() const { return receivedParameter_; }
-
-    pjsip_host_port* getViaAddr() { return &via_addr_; }
-
     /* Returns true if the username and/or hostname match this account */
-    MatchRank matches(const std::string& username, const std::string& hostname) const override;
-
-    /**
-     * Implementation of Account::newOutgoingCall()
-     * Note: keep declaration before newOutgoingCall template.
-     */
-    std::shared_ptr<Call> newOutgoingCall(
-        const std::string& toUrl,
-        const std::map<std::string, std::string>& volatileCallDetails = {}) override;
+    MatchRank matches(std::string_view username, std::string_view hostname) const override;
 
     /**
      * Create outgoing SIPCall.
@@ -250,26 +251,25 @@ public:
      *      The type of this instance is given in template argument.
      *      This type can be any base class of SIPCall class (included).
      */
-#ifndef _MSC_VER
-    template<class T = SIPCall>
-    std::shared_ptr<enable_if_base_of<T, SIPCall>> newOutgoingCall(
-        const std::string& toUrl,
-        const std::map<std::string, std::string>& volatileCallDetails = {});
-#else
-    template<class T>
-    std::shared_ptr<T> newOutgoingCall(
-        const std::string& toUrl,
-        const std::map<std::string, std::string>& volatileCallDetails = {});
-#endif
+    std::shared_ptr<Call> newOutgoingCall(
+        std::string_view toUrl,
+        const std::map<std::string, std::string>& volatileCallDetails = {}) override;
+
+    /**
+     * Create outgoing SIPCall.
+     * @param[in] toUrl The address to call
+     * @param[in] mediaList list of medias
+     * @return A shared pointer on the created call.
+     */
+    std::shared_ptr<Call> newOutgoingCall(std::string_view toUrl,
+                                          const std::vector<MediaAttribute>& mediaList) override;
 
     /**
      * Create incoming SIPCall.
      * @param[in] from The origin of the call
-     * @param details use to set some specific details
-     * @param sipTr: current SIP Transport
-     * @return std::shared_ptr<T> A shared pointer on the created call.
-     *      The type of this instance is given in template argument.
-     *      This type can be any base class of SIPCall class (included).
+     * @param details Call details
+     * @param sipTr: SIP Transport
+     * @return A shared pointer on the created call.
      */
     std::shared_ptr<SIPCall> newIncomingCall(
         const std::string& from,
@@ -284,9 +284,9 @@ public:
 
     virtual bool isSrtpEnabled() const { return true; }
 
-    virtual sip_utils::KeyExchangeProtocol getSrtpKeyExchange() const override
+    virtual KeyExchangeProtocol getSrtpKeyExchange() const override
     {
-        return sip_utils::KeyExchangeProtocol::SDES;
+        return KeyExchangeProtocol::SDES;
     }
 
     virtual bool getSrtpFallback() const override { return false; }
@@ -318,7 +318,6 @@ public:
     std::map<std::string, std::string> getContactDetails(const std::string& uri) const;
 
     void sendTrustRequest(const std::string& to, const std::vector<uint8_t>& payload);
-    void sendTrustRequestConfirm(const std::string& to);
     void sendTextMessage(const std::string& to,
                          const std::map<std::string, std::string>& payloads,
                          uint64_t id,
@@ -366,10 +365,9 @@ public:
         const std::string& peer,
         const DRing::DataTransferId& tid,
         bool isVCard,
-        const std::function<void(PeerConnection*)>& connect_cb,
         const std::function<void(const std::shared_ptr<ChanneledOutgoingTransfer>&)>&
             channeledConnectedCb,
-        const std::function<void()>& onChanneledCancelled);
+        const std::function<void(const std::string&)>& onChanneledCancelled);
 
     ///
     /// Close a E2E connection between a given peer and a given transfer id.
@@ -377,9 +375,7 @@ public:
     /// /// \param[in] peer RingID on request's recipient
     /// /// \param[in] tid linked outgoing data transfer
     ///
-    void closePeerConnection(const std::string& peer, const DRing::DataTransferId& tid);
-
-    std::vector<std::string> publicAddresses();
+    void closePeerConnection(const DRing::DataTransferId& tid);
 
     /// \return true if the given DHT message identifier has been treated
     /// \note if message has not been treated yet this method store this id and returns true at
@@ -437,9 +433,14 @@ public:
     std::map<std::string, std::string> getNearbyPeers() const override;
 
     /**
-     * Add public addresses to ice transport
+     * Store the local/public addresses used to register
      */
-    void registerDhtAddress(IceTransport&);
+    void storeActiveIpAddress(std::function<void()>&& cb = {});
+
+    /**
+     * Create and return ICE options.
+     */
+    void getIceOptions(std::function<void(IceTransportOptions)> cb) noexcept;
 
 #ifdef DRING_TESTABLE
     ConnectionManager& connectionManager() { return *connectionManager_; }
@@ -450,6 +451,117 @@ public:
      * ConnectionManager needs the account to exists
      */
     void shutdownConnections();
+    std::shared_ptr<ChannelSocket> gitSocket(const std::string& deviceId,
+                                             const std::string& conversationId) const
+    {
+        auto deviceSockets = gitSocketList_.find(deviceId);
+        if (deviceSockets == gitSocketList_.end()) {
+            return nullptr;
+        }
+        auto socketIt = deviceSockets->second.find(conversationId);
+        if (socketIt == deviceSockets->second.end()) {
+            return nullptr;
+        }
+        return socketIt->second;
+    }
+
+    void addGitSocket(const std::string& deviceId,
+                      const std::string& conversationId,
+                      const std::shared_ptr<ChannelSocket>& socket)
+    {
+        auto& deviceSockets = gitSocketList_[deviceId];
+        deviceSockets[conversationId] = socket;
+    }
+
+    void removeGitSocket(const std::string& deviceId, const std::string& conversationId)
+    {
+        auto deviceSockets = gitSocketList_.find(deviceId);
+        if (deviceSockets == gitSocketList_.end()) {
+            return;
+        }
+        deviceSockets->second.erase(conversationId);
+        if (deviceSockets->second.empty()) {
+            gitSocketList_.erase(deviceSockets);
+        }
+    }
+
+    std::string_view currentDeviceId() const;
+    // Conversation management
+    std::string startConversation(ConversationMode mode = ConversationMode::INVITES_ONLY,
+                                  const std::string& otherMember = "");
+    void acceptConversationRequest(const std::string& conversationId);
+    void declineConversationRequest(const std::string& conversationId);
+    std::vector<std::string> getConversations();
+    bool removeConversation(const std::string& conversationId);
+    std::vector<std::map<std::string, std::string>> getConversationRequests();
+
+    // Conversation's infos management
+    void updateConversationInfos(const std::string& conversationId,
+                                 const std::map<std::string, std::string>& infos,
+                                 bool sync = true);
+    std::map<std::string, std::string> conversationInfos(const std::string& conversationId) const;
+    std::vector<uint8_t> conversationVCard(const std::string& conversationId) const;
+
+    // Member management
+    bool addConversationMember(const std::string& conversationId,
+                               const std::string& contactUri,
+                               bool sendRequest = true);
+    bool removeConversationMember(const std::string& conversationId,
+                                  const std::string& contactUri,
+                                  bool isDevice = false);
+    std::vector<std::map<std::string, std::string>> getConversationMembers(
+        const std::string& conversationId) const;
+
+    // Message send/load
+    void sendMessage(const std::string& conversationId,
+                     const Json::Value& value,
+                     const std::string& parent = "",
+                     bool announce = true);
+    void sendMessage(const std::string& conversationId,
+                     const std::string& message,
+                     const std::string& parent = "",
+                     const std::string& type = "text/plain",
+                     bool announce = true);
+    /**
+     * Add to the related conversation the call history message
+     * @param uri           Peer number
+     * @param duration_ms   The call duration in ms
+     */
+    void addCallHistoryMessage(const std::string& uri, uint64_t duration_ms);
+    uint32_t loadConversationMessages(const std::string& conversationId,
+                                      const std::string& fromMessage = "",
+                                      size_t n = 0);
+
+    // Received a new commit notification
+    void onNewGitCommit(const std::string& peer,
+                        const std::string& deviceId,
+                        const std::string& conversationId,
+                        const std::string& commitId) override;
+    /**
+     * Pull remote device (do not do it if commitId is already in the current repo)
+     * @param peer              Contact URI
+     * @param deviceId          Contact's device
+     * @param conversationId
+     * @param commitId (optional)
+     */
+    void fetchNewCommits(const std::string& peer,
+                         const std::string& deviceId,
+                         const std::string& conversationId,
+                         const std::string& commitId = "");
+
+    // Invites
+    void onConversationRequest(const std::string& from, const Json::Value&) override;
+    void onNeedConversationRequest(const std::string& from,
+                                   const std::string& conversationId) override;
+
+    void monitor() const;
+
+    /**
+     * Clone a conversation (initial) from device
+     * @param deviceId
+     * @param convId
+     */
+    void cloneConversation(const std::string& deviceId, const std::string& convId);
 
 private:
     NON_COPYABLE(JamiAccount);
@@ -461,6 +573,7 @@ private:
      * Private structures
      */
     struct PendingCall;
+    struct PendingConversationFetch;
     struct PendingMessage;
     struct BuddyInfo;
     struct DiscoveredPeer;
@@ -483,7 +596,9 @@ private:
 
     void startOutgoingCall(const std::shared_ptr<SIPCall>& call, const std::string& toUri);
 
-    void onConnectedOutgoingCall(SIPCall& call, const std::string& to_id, IpAddr target);
+    void onConnectedOutgoingCall(const std::shared_ptr<SIPCall>& call,
+                                 const std::string& to_id,
+                                 IpAddr target);
 
     /**
      * Set the internal state for this account, mainly used to manage account details from the
@@ -531,6 +646,8 @@ private:
 
     void checkPendingCall(const std::string& callId);
     bool handlePendingCall(PendingCall& pc, bool incoming);
+    void forEachPendingCall(const DeviceId& deviceId,
+                            const std::function<void(const std::shared_ptr<SIPCall>&)>& cb);
 
     void loadAccount(const std::string& archive_password = {},
                      const std::string& archive_pin = {},
@@ -586,8 +703,16 @@ private:
      */
     void generateDhParams();
 
+    void loadConvInfos();
+    void saveConvInfos() const;
+
+    void loadConvRequests();
+    void saveConvRequests();
+
     template<class... Args>
     std::shared_ptr<IceTransport> createIceTransport(const Args&... args);
+    void newOutgoingCallHelper(const std::shared_ptr<SIPCall>& call, std::string_view toUri);
+    std::shared_ptr<SIPCall> createSubCall(const std::shared_ptr<SIPCall>& mainCall);
 
 #if HAVE_RINGNS
     std::string nameServer_;
@@ -604,7 +729,7 @@ private:
     /**
      * DHT calls waiting for ICE negotiation
      */
-    std::map<std::string, PendingCall> pendingCalls_;
+    std::map<std::string, PendingCall> pendingCallsDht_;
 
     /**
      * Incoming DHT calls that are not yet actual SIP calls.
@@ -632,6 +757,16 @@ private:
     mutable std::mutex buddyInfoMtx;
     std::map<dht::InfoHash, BuddyInfo> trackedBuddies_;
 
+    /** Conversations */
+    mutable std::mutex conversationsMtx_ {};
+    std::map<std::string, std::shared_ptr<Conversation>> conversations_;
+    bool isConversation(const std::string& convId) const
+    {
+        std::lock_guard<std::mutex> lk(conversationsMtx_);
+        return conversations_.find(convId) != conversations_.end();
+    }
+    mutable std::vector<ConvInfo> convInfos_;
+
     mutable std::mutex dhtValuesMtx_;
     bool dhtPublicInCalls_ {true};
 
@@ -640,17 +775,23 @@ private:
     /**
      * DHT port preference
      */
-    in_port_t dhtPort_ {};
-
-    bool dhtPeerDiscovery_ {false};
+    in_port_t dhtDefaultPort_ {0};
 
     /**
-     * DHT port actually used,
-     * this holds the actual port used for DHT, which may not be the port
-     * selected in the configuration in the case that UPnP is used and the
-     * configured port is already used by another client
+     * DHT port actually used.
+     * This holds the actual DHT port, which might different from the port
+     * set in the configuration. This can be the case if UPnP is used.
      */
-    UsedPort dhtPortUsed_ {};
+    in_port_t dhtPortUsed()
+    {
+        return (upnpCtrl_ and dhtUpnpMapping_.isValid()) ? dhtUpnpMapping_.getExternalPort()
+                                                         : dhtDefaultPort_;
+    }
+
+    /* Current UPNP mapping */
+    upnp::Mapping dhtUpnpMapping_ {upnp::PortType::UDP};
+
+    bool dhtPeerDiscovery_ {false};
 
     /**
      * Proxy
@@ -669,19 +810,9 @@ private:
     bool allowPeersFromContact_ {true};
     bool allowPeersFromTrusted_ {true};
 
-    /**
-     * Optional: "received" parameter from VIA header
-     */
-    std::string receivedParameter_ {};
-
     std::string managerUri_ {};
     std::string managerUsername_ {};
 
-    /**
-     * Optional: "rport" parameter from VIA header
-     */
-    int rPort_ {-1};
-
     /**
      * Optional: via_addr construct from received parameters
      */
@@ -692,7 +823,9 @@ private:
     pjsip_transport* via_tp_ {nullptr};
 
     std::unique_ptr<DhtPeerConnector> dhtPeerConnector_;
+    mutable std::mutex connManagerMtx_ {};
     std::unique_ptr<ConnectionManager> connectionManager_;
+    GitSocketList gitSocketList_ {};
 
     std::mutex discoveryMapMtx_;
     std::shared_ptr<dht::PeerDiscovery> peerDiscovery_;
@@ -711,9 +844,11 @@ private:
      */
     void cacheTurnServers();
 
+    std::chrono::duration<int> turnRefreshDelay_ {std::chrono::seconds(10)};
+
     std::set<std::shared_ptr<dht::http::Request>> requests_;
 
-    std::mutex sipConnectionsMtx_ {};
+    std::mutex sipConnsMtx_ {};
     struct SipConnection
     {
         std::shared_ptr<SipTransport> transport;
@@ -724,12 +859,25 @@ private:
     // NOTE: here we use a vector to avoid race conditions. In fact the contact
     // can ask for a SIP channel when we are creating a new SIP Channel with this
     // peer too.
-    std::map<std::string /* accountId */,
-             std::map<std::string /* deviceId */, std::vector<SipConnection>>>
-        sipConnections_ {};
-    // However, we only negotiate one socket from our side
-    std::set<std::pair<std::string /* accountId */, std::string /* deviceId */>>
-        pendingSipConnections_ {};
+    std::map<SipConnectionKey, std::vector<SipConnection>> sipConns_;
+
+    std::mutex pendingCallsMutex_;
+    std::map<DeviceId, std::vector<std::shared_ptr<SIPCall>>> pendingCalls_;
+
+    std::mutex onConnectionClosedMtx_ {};
+    std::map<DeviceId, std::function<void(const DeviceId&, bool)>> onConnectionClosed_ {};
+    /**
+     * onConnectionClosed contains callbacks that need to be called if a sub call is failing
+     * @param deviceId      The device we are calling
+     * @param eraseDummy    Erase the dummy call (a temporary subcall that must be stop when we will
+     * not create new subcalls)
+     */
+    void callConnectionClosed(const DeviceId& deviceId, bool eraseDummy);
+
+    // Sync connections
+    std::mutex syncConnectionsMtx_;
+    std::map<std::string /* deviceId */, std::vector<std::shared_ptr<ChannelSocket>>>
+        syncConnections_;
 
     /**
      * Ask a device to open a channeled SIP socket
@@ -737,18 +885,38 @@ private:
      * @param deviceId      The device to ask
      * @note triggers cacheSIPConnection
      */
-    void requestSIPConnection(const std::string& peerId, const std::string& deviceId);
+    void requestSIPConnection(const std::string& peerId, const DeviceId& deviceId);
     /**
      * Store a new SIP connection into sipConnections_
-     * @param socket    The new sip channel
+     * @param channel   The new sip channel
      * @param peerId    The contact who owns the device
      * @param deviceId  Device linked to that transport
      */
-    void cacheSIPConnection(std::shared_ptr<ChannelSocket>&& socket,
+    void cacheSIPConnection(std::shared_ptr<ChannelSocket>&& channel,
                             const std::string& peerId,
-                            const std::string& deviceId);
+                            const DeviceId& deviceId);
+    /**
+     * Shutdown a SIP connection
+     * @param channel   The channel to close
+     * @param peerId    The contact who owns the device
+     * @param deviceId  Device linked to that transport
+     */
+    void shutdownSIPConnection(const std::shared_ptr<ChannelSocket>& channel,
+                               const std::string& peerId,
+                               const DeviceId& deviceId);
+
+    /**
+     * Store a new Sync connection
+     * @param socket    The new sync channel
+     * @param peerId    The contact who owns the device
+     * @param deviceId  Device linked to that transport
+     */
+    void cacheSyncConnection(std::shared_ptr<ChannelSocket>&& socket,
+                             const std::string& peerId,
+                             const DeviceId& deviceId);
 
     // File transfers
+    std::mutex transfersMtx_ {};
     std::set<std::string> incomingFileTransfers_ {};
 
     /**
@@ -765,7 +933,7 @@ private:
     bool sendSIPMessage(SipConnection& conn,
                         const std::string& to,
                         void* ctx,
-                        int token,
+                        uint64_t token,
                         const std::map<std::string, std::string>& data,
                         pjsip_endpt_send_callback cb);
 
@@ -779,6 +947,52 @@ private:
      * @param deviceId      Device that will receive the profile
      */
     void sendProfile(const std::string& deviceId);
+
+    // Conversations
+    std::mutex conversationsRequestsMtx_ {};
+    std::map<std::string, ConversationRequest> conversationsRequests_ {};
+    std::mutex pendingConversationsFetchMtx_ {};
+    std::map<std::string, PendingConversationFetch> pendingConversationsFetch_;
+
+    std::mutex gitServersMtx_ {};
+    std::map<dht::Value::Id, std::unique_ptr<GitServer>> gitServers_ {};
+
+    std::shared_ptr<RepeatedTask> conversationsEventHandler {};
+    void checkConversationsEvents();
+    bool handlePendingConversations();
+
+    void syncWith(const std::string& deviceId, const std::shared_ptr<ChannelSocket>& socket);
+    void syncInfos(const std::shared_ptr<ChannelSocket>& socket);
+    void syncWithConnected();
+    std::atomic_bool needsConvSync_ {true};
+
+    /**
+     * Remove a repository and all files
+     * @param convId
+     * @param sync      If we send an update to other account's devices
+     * @param force     True if ignore the removing flag
+     */
+    void removeRepository(const std::string& convId, bool sync, bool force = false);
+
+    /**
+     * Send a message notification to all members
+     * @param conversation
+     * @param commit
+     * @param sync      If we send an update to other account's devices
+     */
+    void sendMessageNotification(const Conversation& conversation,
+                                 const std::string& commitId,
+                                 bool sync);
+
+    void announceMemberMessage(const std::string& convId,
+                               const std::map<std::string, std::string>& message) const;
+
+    /**
+     * Get related conversation with member
+     * @param uri       The member to search for
+     * @return the conversation id if found else empty
+     */
+    std::string getOneToOneConversation(const std::string& uri) const;
 };
 
 static inline std::ostream&
