@@ -1,11 +1,11 @@
 /*
- *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2021 Savoir-faire Linux Inc.
  *
  *  Author: Emmanuel Milou <emmanuel.milou@savoirfairelinux.com>
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *  Author: Андрей Лухнов <aol.nnov@gmail.com>
  *  Author: Mohamed Fenjiro <mohamed.fenjiro@savoirfairelinux.com>
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 3 of the License, or
@@ -31,15 +31,18 @@
 #include "audio/audioloop.h"
 #include "libav_utils.h"
 
+#include <fmt/core.h>
+
 #include <thread>
 #include <atomic>
 #include <chrono>
 
 namespace jami {
 
-class AlsaThread {
+class AlsaThread
+{
 public:
-    AlsaThread(AlsaLayer *alsa);
+    AlsaThread(AlsaLayer* alsa);
     ~AlsaThread();
     void start();
     bool isRunning() const;
@@ -52,11 +55,14 @@ private:
     std::thread thread_;
 };
 
-AlsaThread::AlsaThread(AlsaLayer *alsa)
-    : alsa_(alsa), running_(false), thread_()
+AlsaThread::AlsaThread(AlsaLayer* alsa)
+    : alsa_(alsa)
+    , running_(false)
+    , thread_()
 {}
 
-bool AlsaThread::isRunning() const
+bool
+AlsaThread::isRunning() const
 {
     return running_;
 }
@@ -68,18 +74,20 @@ AlsaThread::~AlsaThread()
         thread_.join();
 }
 
-void AlsaThread::start()
+void
+AlsaThread::start()
 {
     running_ = true;
     thread_ = std::thread(&AlsaThread::run, this);
 }
 
-void AlsaThread::run()
+void
+AlsaThread::run()
 {
-    alsa_->run();
+    alsa_->run(running_);
 }
 
-AlsaLayer::AlsaLayer(const AudioPreference &pref)
+AlsaLayer::AlsaLayer(const AudioPreference& pref)
     : AudioLayer(pref)
     , indexIn_(pref.getAlsaCardin())
     , indexOut_(pref.getAlsaCardout())
@@ -90,98 +98,60 @@ AlsaLayer::AlsaLayer(const AudioPreference &pref)
     , audioPlugin_(pref.getAlsaPlugin())
     , playbackBuff_(0, audioFormat_)
     , captureBuff_(0, audioFormat_)
-    , playbackIBuff_(1024)
-    , captureIBuff_(1024)
-    , is_playback_prepared_(false)
     , is_capture_prepared_(false)
     , is_playback_running_(false)
     , is_capture_running_(false)
     , is_playback_open_(false)
     , is_capture_open_(false)
     , audioThread_(nullptr)
-{}
+{
+    setHasNativeAEC(false);
+}
 
 AlsaLayer::~AlsaLayer()
 {
+    status_ = Status::Idle;
     audioThread_.reset();
 
     /* Then close the audio devices */
     closeCaptureStream();
     closePlaybackStream();
-}
-
-void AlsaLayer::initAudioLayer()
-{
-    std::string pcmp;
-    std::string pcmr;
-    std::string pcmc;
-
-    if (audioPlugin_ == PCM_DMIX_DSNOOP) {
-        pcmp = buildDeviceTopo(PCM_DMIX, indexOut_);
-        pcmr = buildDeviceTopo(PCM_DMIX, indexRing_);
-        pcmc = buildDeviceTopo(PCM_DSNOOP, indexIn_);
-    } else {
-        pcmp = buildDeviceTopo(audioPlugin_, indexOut_);
-        pcmr = buildDeviceTopo(audioPlugin_, indexRing_);
-        pcmc = buildDeviceTopo(audioPlugin_, indexIn_);
-    }
-
-    if (not is_capture_open_) {
-        is_capture_open_ = openDevice(&captureHandle_, pcmc, SND_PCM_STREAM_CAPTURE, audioInputFormat_);
-
-        if (not is_capture_open_)
-            emitSignal<DRing::ConfigurationSignal::Error>(ALSA_CAPTURE_DEVICE);
-    }
-
-    if (not is_playback_open_) {
-        is_playback_open_ = openDevice(&playbackHandle_, pcmp, SND_PCM_STREAM_PLAYBACK, audioFormat_);
-
-        if (not is_playback_open_)
-            emitSignal<DRing::ConfigurationSignal::Error>(ALSA_PLAYBACK_DEVICE);
-
-        if (getIndexPlayback() != getIndexRingtone())
-            if (!openDevice(&ringtoneHandle_, pcmr, SND_PCM_STREAM_PLAYBACK, audioFormat_))
-                emitSignal<DRing::ConfigurationSignal::Error>(ALSA_PLAYBACK_DEVICE);
-    }
-
-    hardwareFormatAvailable(getFormat());
-    hardwareInputFormatAvailable(audioInputFormat_);
-
-    prepareCaptureStream();
-    preparePlaybackStream();
-
-    startCaptureStream();
-    startPlaybackStream();
-
-    flushMain();
-    flushUrgent();
+    closeRingtoneStream();
 }
 
 /**
  * Reimplementation of run()
  */
-void AlsaLayer::run()
+void
+AlsaLayer::run(const std::atomic<bool>& isRunning)
 {
-    initAudioLayer();
-    {
-        std::lock_guard<std::mutex> lock(mutex_);
-        status_ = AudioLayer::Status::Started;
-    }
-    startedCv_.notify_all();
+    if (playbackHandle_)
+        playbackChanged(true);
+    if (captureHandle_)
+        recordChanged(true);
 
-    while (status_ == AudioLayer::Status::Started and audioThread_ and audioThread_->isRunning()) {
+    while (status_ == Status::Started and isRunning) {
         playback();
         ringtone();
         capture();
     }
+
+    playbackChanged(false);
+    recordChanged(false);
 }
 
 // Retry approach taken from pa_linux_alsa.c, part of PortAudio
-bool AlsaLayer::openDevice(snd_pcm_t **pcm, const std::string &dev, snd_pcm_stream_t stream, AudioFormat& format)
+bool
+AlsaLayer::openDevice(snd_pcm_t** pcm,
+                      const std::string& dev,
+                      snd_pcm_stream_t stream,
+                      AudioFormat& format)
 {
-    JAMI_DBG("Alsa: Opening %s",  dev.c_str());
+    JAMI_DBG("Alsa: Opening %s device '%s'",
+        (stream == SND_PCM_STREAM_CAPTURE) ? "capture" : "playback",
+        dev.c_str());
 
-    static const int MAX_RETRIES = 20; // times of 100ms
+    static const int MAX_RETRIES = 10; // times of 100ms
     int err, tries = 0;
     do {
         err = snd_pcm_open(pcm, dev.c_str(), stream, 0);
@@ -189,18 +159,17 @@ bool AlsaLayer::openDevice(snd_pcm_t **pcm, const std::string &dev, snd_pcm_stre
         if (err == -EBUSY) {
             // We're called in audioThread_ context, so if exit is requested
             // force return now
-            if ((not audioThread_) or (not audioThread_->isRunning()))
-                return false;
             std::this_thread::sleep_for(std::chrono::milliseconds(100));
         }
     } while (err == -EBUSY and ++tries <= MAX_RETRIES);
 
     if (err < 0) {
         JAMI_ERR("Alsa: couldn't open %s device %s : %s",
-              (stream == SND_PCM_STREAM_CAPTURE)? "capture" :
-              (stream == SND_PCM_STREAM_PLAYBACK)? "playback" : "ringtone",
-              dev.c_str(),
-              snd_strerror(err));
+                 (stream == SND_PCM_STREAM_CAPTURE)
+                     ? "capture"
+                     : (stream == SND_PCM_STREAM_PLAYBACK) ? "playback" : "ringtone",
+                 dev.c_str(),
+                 snd_strerror(err));
         return false;
     }
 
@@ -212,46 +181,75 @@ bool AlsaLayer::openDevice(snd_pcm_t **pcm, const std::string &dev, snd_pcm_stre
     return true;
 }
 
-void
-AlsaLayer::startStream(AudioStreamType stream)
+void AlsaLayer::startStream(AudioDeviceType type)
 {
-    {
-        std::lock_guard<std::mutex> lock(mutex_);
-        if (status_ != Status::Idle)
-            return;
-        status_ = Status::Starting;
+    status_ = Status::Starting;
+    audioThread_.reset();
+
+    bool dsnop = audioPlugin_ == PCM_DMIX_DSNOOP;
+
+    if (type == AudioDeviceType::PLAYBACK and not is_playback_open_) {
+        is_playback_open_ = openDevice(&playbackHandle_,
+                                       buildDeviceTopo(dsnop ? PCM_DMIX : audioPlugin_, indexOut_),
+                                       SND_PCM_STREAM_PLAYBACK,
+                                       audioFormat_);
+        if (not is_playback_open_)
+            emitSignal<DRing::ConfigurationSignal::Error>(ALSA_PLAYBACK_DEVICE);
+
+        hardwareFormatAvailable(getFormat());
+        startPlaybackStream();
     }
 
-    dcblocker_.reset();
-
-    if (is_playback_running_ and is_capture_running_)
-        return;
-
-    if (not audioThread_) {
-        audioThread_.reset(new AlsaThread(this));
-        audioThread_->start();
-    } else if (!audioThread_->isRunning()) {
-        audioThread_->start();
+    if (type == AudioDeviceType::RINGTONE and getIndexPlayback() != getIndexRingtone() and not ringtoneHandle_) {
+        if (!openDevice(&ringtoneHandle_,
+                        buildDeviceTopo(dsnop ? PCM_DMIX : audioPlugin_, indexRing_),
+                        SND_PCM_STREAM_PLAYBACK,
+                        audioFormat_))
+            emitSignal<DRing::ConfigurationSignal::Error>(ALSA_PLAYBACK_DEVICE);
     }
+
+    if (type == AudioDeviceType::CAPTURE and not is_capture_open_) {
+        is_capture_open_ = openDevice(&captureHandle_,
+                                      buildDeviceTopo(dsnop ? PCM_DSNOOP : audioPlugin_, indexIn_),
+                                      SND_PCM_STREAM_CAPTURE,
+                                      audioInputFormat_);
+
+        if (not is_capture_open_)
+            emitSignal<DRing::ConfigurationSignal::Error>(ALSA_CAPTURE_DEVICE);
+        prepareCaptureStream();
+        startCaptureStream();
+    }
+
+    status_ = Status::Started;
+    audioThread_.reset(new AlsaThread(this));
+    audioThread_->start();
 }
 
 void
-AlsaLayer::stopStream()
+AlsaLayer::stopStream(AudioDeviceType stream)
 {
     audioThread_.reset();
 
-    closeCaptureStream();
-    closePlaybackStream();
+    if (stream == AudioDeviceType::CAPTURE && is_capture_open_) {
+        closeCaptureStream();
+    }
 
-    playbackHandle_ = nullptr;
-    captureHandle_ = nullptr;
-    ringtoneHandle_ = nullptr;
+    if (stream == AudioDeviceType::PLAYBACK && is_playback_open_) {
+        closePlaybackStream();
+        flushUrgent();
+        flushMain();
+    }
 
-    /* Flush the ring buffers */
-    flushUrgent();
-    flushMain();
+    if (stream == AudioDeviceType::RINGTONE and ringtoneHandle_) {
+        closeRingtoneStream();
+    }
 
-    status_ = Status::Idle;
+    if (is_capture_open_ or is_playback_open_ or ringtoneHandle_) {
+        audioThread_.reset(new AlsaThread(this));
+        audioThread_->start();
+    } else {
+        status_ = Status::Idle;
+    }
 }
 
 /*
@@ -262,14 +260,16 @@ AlsaLayer::stopStream()
  *  display an error if the function failed
  *  return the function return value
  */
-#define ALSA_CALL(call, error) ({ \
+#define ALSA_CALL(call, error) \
+    ({ \
         int err_code = call; \
         if (err_code < 0) \
             JAMI_ERR(error ": %s", snd_strerror(err_code)); \
-            err_code; \
-        })
+        err_code; \
+    })
 
-void AlsaLayer::stopCaptureStream()
+void
+AlsaLayer::stopCaptureStream()
 {
     if (captureHandle_ && ALSA_CALL(snd_pcm_drop(captureHandle_), "couldn't stop capture") >= 0) {
         is_capture_running_ = false;
@@ -277,76 +277,85 @@ void AlsaLayer::stopCaptureStream()
     }
 }
 
-void AlsaLayer::closeCaptureStream()
+void
+AlsaLayer::closeCaptureStream()
 {
     if (is_capture_prepared_ and is_capture_running_)
         stopCaptureStream();
 
-    if (is_capture_open_ && ALSA_CALL(snd_pcm_close(captureHandle_), "Couldn't close capture") >= 0)
+    JAMI_DBG("Alsa: Closing capture stream");
+    if (is_capture_open_ && ALSA_CALL(snd_pcm_close(captureHandle_), "Couldn't close capture") >= 0) {
         is_capture_open_ = false;
+        captureHandle_ = nullptr;
+    }
 }
 
-void AlsaLayer::startCaptureStream()
+void
+AlsaLayer::startCaptureStream()
 {
     if (captureHandle_ and not is_capture_running_)
         if (ALSA_CALL(snd_pcm_start(captureHandle_), "Couldn't start capture") >= 0)
             is_capture_running_ = true;
 }
 
-void AlsaLayer::stopPlaybackStream()
+void
+AlsaLayer::stopPlaybackStream()
 {
-    if (ringtoneHandle_ and is_playback_running_)
-        ALSA_CALL(snd_pcm_drop(ringtoneHandle_), "Couldn't stop ringtone");
-
     if (playbackHandle_ and is_playback_running_) {
         if (ALSA_CALL(snd_pcm_drop(playbackHandle_), "Couldn't stop playback") >= 0) {
             is_playback_running_ = false;
-            is_playback_prepared_ = false;
         }
     }
 }
 
-
-void AlsaLayer::closePlaybackStream()
+void
+AlsaLayer::closePlaybackStream()
 {
-    if (is_playback_prepared_ and is_playback_running_)
+    if (is_playback_running_)
         stopPlaybackStream();
 
     if (is_playback_open_) {
-        if (ringtoneHandle_)
-            ALSA_CALL(snd_pcm_close(ringtoneHandle_), "Couldn't stop ringtone");
-
+        JAMI_DBG("Alsa: Closing playback stream");
         if (ALSA_CALL(snd_pcm_close(playbackHandle_), "Coulnd't close playback") >= 0)
             is_playback_open_ = false;
+        playbackHandle_ = nullptr;
     }
-
 }
 
-void AlsaLayer::startPlaybackStream()
+void
+AlsaLayer::closeRingtoneStream()
+{
+    if (ringtoneHandle_) {
+        ALSA_CALL(snd_pcm_drop(ringtoneHandle_), "Couldn't stop ringtone");
+        ALSA_CALL(snd_pcm_close(ringtoneHandle_), "Couldn't close ringtone");
+        ringtoneHandle_ = nullptr;
+    }
+}
+
+void
+AlsaLayer::startPlaybackStream()
 {
     is_playback_running_ = true;
 }
 
-void AlsaLayer::prepareCaptureStream()
+void
+AlsaLayer::prepareCaptureStream()
 {
     if (is_capture_open_ and not is_capture_prepared_)
         if (ALSA_CALL(snd_pcm_prepare(captureHandle_), "Couldn't prepare capture") >= 0)
             is_capture_prepared_ = true;
 }
 
-void AlsaLayer::preparePlaybackStream()
+bool
+AlsaLayer::alsa_set_params(snd_pcm_t* pcm_handle, AudioFormat& format)
 {
-    is_playback_prepared_ = true;
-}
+#define TRY(call, error) \
+    do { \
+        if (ALSA_CALL(call, error) < 0) \
+            return false; \
+    } while (0)
 
-bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle, AudioFormat& format)
-{
-#define TRY(call, error) do { \
-    if (ALSA_CALL(call, error) < 0) \
-    return false; \
-} while(0)
-
-    snd_pcm_hw_params_t *hwparams;
+    snd_pcm_hw_params_t* hwparams;
     snd_pcm_hw_params_alloca(&hwparams);
 
     const unsigned RING_ALSA_PERIOD_SIZE = 160;
@@ -357,10 +366,10 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle, AudioFormat& format)
     snd_pcm_uframes_t buffer_size = RING_ALSA_BUFFER_SIZE;
     unsigned int periods = RING_ALSA_NB_PERIOD;
 
-    snd_pcm_uframes_t  period_size_min = 0;
-    snd_pcm_uframes_t  period_size_max = 0;
-    snd_pcm_uframes_t  buffer_size_min = 0;
-    snd_pcm_uframes_t  buffer_size_max = 0;
+    snd_pcm_uframes_t period_size_min = 0;
+    snd_pcm_uframes_t period_size_max = 0;
+    snd_pcm_uframes_t buffer_size_min = 0;
+    snd_pcm_uframes_t buffer_size_max = 0;
 
 #define HW pcm_handle, hwparams /* hardware parameters */
     TRY(snd_pcm_hw_params_any(HW), "hwparams init");
@@ -368,7 +377,8 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle, AudioFormat& format)
     TRY(snd_pcm_hw_params_set_access(HW, SND_PCM_ACCESS_RW_INTERLEAVED), "access type");
     TRY(snd_pcm_hw_params_set_format(HW, SND_PCM_FORMAT_S16_LE), "sample format");
 
-    TRY(snd_pcm_hw_params_set_rate_resample(HW, 0), "hardware sample rate"); /* prevent software resampling */
+    TRY(snd_pcm_hw_params_set_rate_resample(HW, 0),
+        "hardware sample rate"); /* prevent software resampling */
     TRY(snd_pcm_hw_params_set_rate_near(HW, &format.sample_rate, nullptr), "sample rate");
 
     // TODO: use snd_pcm_query_chmaps or similar to get hardware channel num
@@ -387,9 +397,12 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle, AudioFormat& format)
     period_size = period_size > period_size_max ? period_size_max : period_size;
     period_size = period_size < period_size_min ? period_size_min : period_size;
 
-    TRY(snd_pcm_hw_params_set_buffer_size_near(HW, &buffer_size), "Unable to set buffer size for playback");
-    TRY(snd_pcm_hw_params_set_period_size_near(HW, &period_size, nullptr), "Unable to set period size for playback");
-    TRY(snd_pcm_hw_params_set_periods_near(HW, &periods, nullptr), "Unable to set number of periods for playback");
+    TRY(snd_pcm_hw_params_set_buffer_size_near(HW, &buffer_size),
+        "Unable to set buffer size for playback");
+    TRY(snd_pcm_hw_params_set_period_size_near(HW, &period_size, nullptr),
+        "Unable to set period size for playback");
+    TRY(snd_pcm_hw_params_set_periods_near(HW, &periods, nullptr),
+        "Unable to set number of periods for playback");
     TRY(snd_pcm_hw_params(HW), "hwparams");
 
     snd_pcm_hw_params_get_buffer_size(hwparams, &buffer_size);
@@ -407,10 +420,10 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle, AudioFormat& format)
 #undef HW
 
     JAMI_DBG("%s using format %s",
-          (snd_pcm_stream(pcm_handle) == SND_PCM_STREAM_PLAYBACK) ? "playback" : "capture",
-          format.toString().c_str() );
+             (snd_pcm_stream(pcm_handle) == SND_PCM_STREAM_PLAYBACK) ? "playback" : "capture",
+             format.toString().c_str());
 
-    snd_pcm_sw_params_t *swparams = nullptr;
+    snd_pcm_sw_params_t* swparams = nullptr;
     snd_pcm_sw_params_alloca(&swparams);
 
 #define SW pcm_handle, swparams /* software parameters */
@@ -427,9 +440,11 @@ bool AlsaLayer::alsa_set_params(snd_pcm_t *pcm_handle, AudioFormat& format)
 // TODO first frame causes broken pipe (underrun) because not enough data is sent
 // we should wait until the handle is ready
 void
-AlsaLayer::write(const AudioFrame& buffer, snd_pcm_t * handle)
+AlsaLayer::write(const AudioFrame& buffer, snd_pcm_t* handle)
 {
-    int err = snd_pcm_writei(handle, (const void*)buffer.pointer()->data[0], buffer.pointer()->nb_samples);
+    int err = snd_pcm_writei(handle,
+                             (const void*) buffer.pointer()->data[0],
+                             buffer.pointer()->nb_samples);
 
     if (err < 0)
         snd_pcm_recover(handle, err, 0);
@@ -438,48 +453,49 @@ AlsaLayer::write(const AudioFrame& buffer, snd_pcm_t * handle)
         return;
 
     switch (err) {
+    case -EPIPE:
+    case -ESTRPIPE:
+    case -EIO: {
+        snd_pcm_status_t* status;
+        snd_pcm_status_alloca(&status);
 
-        case -EPIPE:
-        case -ESTRPIPE:
-        case -EIO: {
-            snd_pcm_status_t* status;
-            snd_pcm_status_alloca(&status);
-
-            if (ALSA_CALL(snd_pcm_status(handle, status), "Cannot get playback handle status") >= 0)
-                if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
-                    stopPlaybackStream();
-                    preparePlaybackStream();
-                    startPlaybackStream();
-                }
-
-            ALSA_CALL(snd_pcm_writei(handle, (const void*)buffer.pointer()->data[0], buffer.pointer()->nb_samples), "XRUN handling failed");
-            break;
-        }
-
-        case -EBADFD: {
-            snd_pcm_status_t* status;
-            snd_pcm_status_alloca(&status);
-
-            if (ALSA_CALL(snd_pcm_status(handle, status), "Cannot get playback handle status") >= 0) {
-                if (snd_pcm_status_get_state(status) == SND_PCM_STATE_SETUP) {
-                    JAMI_ERR("Writing in state SND_PCM_STATE_SETUP, should be "
-                          "SND_PCM_STATE_PREPARED or SND_PCM_STATE_RUNNING");
-                    int error = snd_pcm_prepare(handle);
-
-                    if (error < 0) {
-                        JAMI_ERR("Failed to prepare handle: %s", snd_strerror(error));
-                        stopPlaybackStream();
-                    }
-                }
+        if (ALSA_CALL(snd_pcm_status(handle, status), "Cannot get playback handle status") >= 0)
+            if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
+                stopPlaybackStream();
+                startPlaybackStream();
             }
 
-            break;
+        ALSA_CALL(snd_pcm_writei(handle,
+                                 (const void*) buffer.pointer()->data[0],
+                                 buffer.pointer()->nb_samples),
+                  "XRUN handling failed");
+        break;
+    }
+
+    case -EBADFD: {
+        snd_pcm_status_t* status;
+        snd_pcm_status_alloca(&status);
+
+        if (ALSA_CALL(snd_pcm_status(handle, status), "Cannot get playback handle status") >= 0) {
+            if (snd_pcm_status_get_state(status) == SND_PCM_STATE_SETUP) {
+                JAMI_ERR("Writing in state SND_PCM_STATE_SETUP, should be "
+                         "SND_PCM_STATE_PREPARED or SND_PCM_STATE_RUNNING");
+                int error = snd_pcm_prepare(handle);
+
+                if (error < 0) {
+                    JAMI_ERR("Failed to prepare handle: %s", snd_strerror(error));
+                    stopPlaybackStream();
+                }
+            }
         }
 
-        default:
-            JAMI_ERR("Unknown write error, dropping frames: %s", snd_strerror(err));
-            stopPlaybackStream();
-            break;
+        break;
+    }
+
+    default:
+        JAMI_ERR("Unknown write error, dropping frames: %s", snd_strerror(err));
+        stopPlaybackStream();
+        break;
     }
 }
 
@@ -500,49 +516,44 @@ AlsaLayer::read(unsigned frames)
     }
 
     switch (err) {
-        case -EPIPE:
-        case -ESTRPIPE:
-        case -EIO: {
-            snd_pcm_status_t* status;
-            snd_pcm_status_alloca(&status);
+    case -EPIPE:
+    case -ESTRPIPE:
+    case -EIO: {
+        snd_pcm_status_t* status;
+        snd_pcm_status_alloca(&status);
 
-            if (ALSA_CALL(snd_pcm_status(captureHandle_, status), "Get status failed") >= 0)
-                if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
-                    stopCaptureStream();
-                    prepareCaptureStream();
-                    startCaptureStream();
-                }
+        if (ALSA_CALL(snd_pcm_status(captureHandle_, status), "Get status failed") >= 0)
+            if (snd_pcm_status_get_state(status) == SND_PCM_STATE_XRUN) {
+                stopCaptureStream();
+                prepareCaptureStream();
+                startCaptureStream();
+            }
 
-            JAMI_ERR("XRUN capture ignored (%s)", snd_strerror(err));
-            break;
-        }
-
-        case -EPERM:
-            JAMI_ERR("Can't capture, EPERM (%s)", snd_strerror(err));
-            prepareCaptureStream();
-            startCaptureStream();
-            break;
+        JAMI_ERR("XRUN capture ignored (%s)", snd_strerror(err));
+        break;
     }
 
-    return 0;
+    case -EPERM:
+        JAMI_ERR("Can't capture, EPERM (%s)", snd_strerror(err));
+        prepareCaptureStream();
+        startCaptureStream();
+        break;
+    }
+
+    return {};
 }
 
 std::string
-AlsaLayer::buildDeviceTopo(const std::string &plugin, int card)
+AlsaLayer::buildDeviceTopo(const std::string& plugin, int card)
 {
-    std::stringstream ss;
-    std::string pcm(plugin);
+    if (plugin == PCM_DEFAULT)
+        return plugin;
 
-    if (pcm == PCM_DEFAULT)
-        return pcm;
-
-    ss << ":" << card;
-
-    return pcm + ss.str();
+    return fmt::format("{}:{}", plugin, card);
 }
 
 static bool
-safeUpdate(snd_pcm_t *handle, long &samples)
+safeUpdate(snd_pcm_t* handle, long& samples)
 {
     samples = snd_pcm_avail_update(handle);
 
@@ -559,12 +570,12 @@ safeUpdate(snd_pcm_t *handle, long &samples)
 }
 
 static std::vector<std::string>
-getValues(const std::vector<HwIDPair> &deviceMap)
+getValues(const std::vector<HwIDPair>& deviceMap)
 {
     std::vector<std::string> audioDeviceList;
     audioDeviceList.reserve(deviceMap.size());
 
-    for (const auto & dev : deviceMap)
+    for (const auto& dev : deviceMap)
         audioDeviceList.push_back(dev.second);
 
     return audioDeviceList;
@@ -586,7 +597,7 @@ std::vector<HwIDPair>
 AlsaLayer::getAudioDeviceIndexMap(bool getCapture) const
 {
     snd_ctl_t* handle;
-    snd_ctl_card_info_t *info;
+    snd_ctl_card_info_t* info;
     snd_pcm_info_t* pcminfo;
     snd_ctl_card_info_alloca(&info);
     snd_pcm_info_alloca(&pcminfo);
@@ -599,32 +610,32 @@ AlsaLayer::getAudioDeviceIndexMap(bool getCapture) const
         return audioDevice;
 
     do {
-        std::stringstream ss;
-        ss << numCard;
-        std::string name = "hw:" + ss.str();
+        std::string name = fmt::format("hw:{}", numCard);
 
         if (snd_ctl_open(&handle, name.c_str(), 0) == 0) {
             if (snd_ctl_card_info(handle, info) == 0) {
                 snd_pcm_info_set_device(pcminfo, 0);
-                snd_pcm_info_set_stream(pcminfo, getCapture ? SND_PCM_STREAM_CAPTURE : SND_PCM_STREAM_PLAYBACK);
+                snd_pcm_info_set_stream(pcminfo,
+                                        getCapture ? SND_PCM_STREAM_CAPTURE
+                                                   : SND_PCM_STREAM_PLAYBACK);
 
                 int err;
-
                 if ((err = snd_ctl_pcm_info(handle, pcminfo)) < 0) {
-                    JAMI_WARN("Cannot get info for %s %s: %s", getCapture ?
-                         "capture device" : "playback device", name.c_str(),
-                         snd_strerror(err));
+                    JAMI_WARN("Cannot get info for %s %s: %s",
+                              getCapture ? "capture device" : "playback device",
+                              name.c_str(),
+                              snd_strerror(err));
                 } else {
                     JAMI_DBG("card %i : %s [%s]",
-                          numCard,
-                          snd_ctl_card_info_get_id(info),
-                          snd_ctl_card_info_get_name(info));
+                             numCard,
+                             snd_ctl_card_info_get_id(info),
+                             snd_ctl_card_info_get_name(info));
                     std::string description = snd_ctl_card_info_get_name(info);
                     description.append(" - ");
                     description.append(snd_pcm_info_get_name(pcminfo));
 
                     // The number of the sound card is associated with a string description
-                    audioDevice.push_back(HwIDPair(numCard, description));
+                    audioDevice.emplace_back(numCard, std::move(description));
                 }
             }
 
@@ -632,15 +643,13 @@ AlsaLayer::getAudioDeviceIndexMap(bool getCapture) const
         }
     } while (snd_card_next(&numCard) >= 0 && numCard >= 0);
 
-
     return audioDevice;
 }
 
-
 bool
-AlsaLayer::soundCardIndexExists(int card, DeviceType stream)
+AlsaLayer::soundCardIndexExists(int card, AudioDeviceType stream)
 {
-    const std::string name("hw:" + std::to_string(card));
+    std::string name = fmt::format("hw:{}", card);
 
     snd_ctl_t* handle;
     if (snd_ctl_open(&handle, name.c_str(), 0) != 0)
@@ -648,18 +657,20 @@ AlsaLayer::soundCardIndexExists(int card, DeviceType stream)
 
     snd_pcm_info_t* pcminfo;
     snd_pcm_info_alloca(&pcminfo);
-    snd_pcm_info_set_stream(pcminfo, stream == DeviceType::PLAYBACK ?  SND_PCM_STREAM_PLAYBACK : SND_PCM_STREAM_CAPTURE);
+    snd_pcm_info_set_stream(pcminfo,
+                            stream == AudioDeviceType::PLAYBACK ? SND_PCM_STREAM_PLAYBACK
+                                                                : SND_PCM_STREAM_CAPTURE);
     bool ret = snd_ctl_pcm_info(handle, pcminfo) >= 0;
     snd_ctl_close(handle);
     return ret;
 }
 
 int
-AlsaLayer::getAudioDeviceIndex(const std::string &description, DeviceType type) const
+AlsaLayer::getAudioDeviceIndex(const std::string& description, AudioDeviceType type) const
 {
-    std::vector<HwIDPair> devices = getAudioDeviceIndexMap(type == DeviceType::CAPTURE);
+    std::vector<HwIDPair> devices = getAudioDeviceIndexMap(type == AudioDeviceType::CAPTURE);
 
-    for (const auto & dev : devices)
+    for (const auto& dev : devices)
         if (dev.second == description)
             return dev.first;
 
@@ -668,30 +679,33 @@ AlsaLayer::getAudioDeviceIndex(const std::string &description, DeviceType type)
 }
 
 std::string
-AlsaLayer::getAudioDeviceName(int index, DeviceType type) const
+AlsaLayer::getAudioDeviceName(int index, AudioDeviceType type) const
 {
     // a bit ugly and wrong.. i do not know how to implement it better in alsalayer.
-    // in addition, for now it is used in pulselayer only due to alsa and pulse layers api differences.
-    // but after some tweaking in alsalayer, it could be used in it too.
+    // in addition, for now it is used in pulselayer only due to alsa and pulse layers api
+    // differences. but after some tweaking in alsalayer, it could be used in it too.
     switch (type) {
-        case DeviceType::PLAYBACK:
-        case DeviceType::RINGTONE:
-            return getPlaybackDeviceList().at(index);
+    case AudioDeviceType::PLAYBACK:
+    case AudioDeviceType::RINGTONE:
+        return getPlaybackDeviceList().at(index);
 
-        case DeviceType::CAPTURE:
-            return getCaptureDeviceList().at(index);
-        default:
-            // Should never happen
-            JAMI_ERR("Unexpected type");
-            return "";
+    case AudioDeviceType::CAPTURE:
+        return getCaptureDeviceList().at(index);
+    default:
+        // Should never happen
+        JAMI_ERR("Unexpected type");
+        return "";
     }
 }
 
-void AlsaLayer::capture()
+void
+AlsaLayer::capture()
 {
     if (!captureHandle_ or !is_capture_running_)
         return;
 
+    snd_pcm_wait(captureHandle_, 10);
+
     int toGetFrames = snd_pcm_avail_update(captureHandle_);
     if (toGetFrames < 0)
         JAMI_ERR("Audio: Mic error: %s", snd_strerror(toGetFrames));
@@ -706,7 +720,8 @@ void AlsaLayer::capture()
         JAMI_ERR("ALSA MIC : Couldn't read!");
 }
 
-void AlsaLayer::playback()
+void
+AlsaLayer::playback()
 {
     if (!playbackHandle_)
         return;
@@ -722,7 +737,8 @@ void AlsaLayer::playback()
     }
 }
 
-void AlsaLayer::ringtone()
+void
+AlsaLayer::ringtone()
 {
     if (!ringtoneHandle_)
         return;
@@ -736,23 +752,24 @@ void AlsaLayer::ringtone()
     }
 }
 
-void AlsaLayer::updatePreference(AudioPreference &preference, int index, DeviceType type)
+void
+AlsaLayer::updatePreference(AudioPreference& preference, int index, AudioDeviceType type)
 {
     switch (type) {
-        case DeviceType::PLAYBACK:
-            preference.setAlsaCardout(index);
-            break;
+    case AudioDeviceType::PLAYBACK:
+        preference.setAlsaCardout(index);
+        break;
 
-        case DeviceType::CAPTURE:
-            preference.setAlsaCardin(index);
-            break;
+    case AudioDeviceType::CAPTURE:
+        preference.setAlsaCardin(index);
+        break;
 
-        case DeviceType::RINGTONE:
-            preference.setAlsaCardring(index);
-            break;
+    case AudioDeviceType::RINGTONE:
+        preference.setAlsaCardring(index);
+        break;
 
-        default:
-            break;
+    default:
+        break;
     }
 }
 
