@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2021 Savoir-faire Linux Inc.
  *
  *  Author: Tristan Matthews <tristan.matthews@savoirfairelinux.com>
  *  Author: Philippe Gorley <philippe.gorley@savoirfairelinux.com>
@@ -49,8 +49,12 @@
 namespace jami {
 
 AudioRtpSession::AudioRtpSession(const std::string& id)
-    : RtpSession(id)
+    : RtpSession(id, MediaType::MEDIA_AUDIO)
+    , rtcpCheckerThread_([] { return true; }, [this] { processRtcpChecker(); }, [] {})
+
 {
+    JAMI_DBG("Created Audio RTP session: %p - call Id %s", this, callID_.c_str());
+
     // don't move this into the initializer list or Cthulus will emerge
     ringbuffer_ = Manager::instance().getRingBufferPool().createRingBuffer(callID_);
 }
@@ -58,16 +62,23 @@ AudioRtpSession::AudioRtpSession(const std::string& id)
 AudioRtpSession::~AudioRtpSession()
 {
     stop();
+    JAMI_DBG("Destroyed Audio RTP session: %p - call Id %s", this, callID_.c_str());
 }
 
 void
 AudioRtpSession::startSender()
 {
-    if (not send_.enabled or send_.holding) {
+    JAMI_DBG("Start audio RTP sender: input [%s] - muted [%s]",
+             input_.c_str(),
+             muteState_ ? "YES" : "NO");
+
+    if (not send_.enabled or send_.onHold) {
         JAMI_WARN("Audio sending disabled");
         if (sender_) {
             if (socketPair_)
                 socketPair_->interrupt();
+            if (audioInput_)
+                audioInput_->detach(sender_.get());
             sender_.reset();
         }
         return;
@@ -75,13 +86,17 @@ AudioRtpSession::startSender()
 
     if (sender_)
         JAMI_WARN("Restarting audio sender");
+    if (audioInput_)
+        audioInput_->detach(sender_.get());
 
     // sender sets up input correctly, we just keep a reference in case startSender is called
     audioInput_ = jami::getAudioInput(callID_);
+    audioInput_->setMuted(muteState_);
+    audioInput_->setSuccessfulSetupCb(onSuccessfulSetup_);
     auto newParams = audioInput_->switchInput(input_);
     try {
-        if (newParams.valid() &&
-            newParams.wait_for(NEWPARAMS_TIMEOUT) == std::future_status::ready) {
+        if (newParams.valid()
+            && newParams.wait_for(NEWPARAMS_TIMEOUT) == std::future_status::ready) {
             localAudioParams_ = newParams.get();
         } else {
             JAMI_ERR() << "No valid new audio parameters";
@@ -92,6 +107,8 @@ AudioRtpSession::startSender()
         return;
     }
 
+    send_.fecEnabled = true;
+
     // be sure to not send any packets before saving last RTP seq value
     socketPair_->stopSendOp();
     if (sender_)
@@ -99,12 +116,21 @@ AudioRtpSession::startSender()
     try {
         sender_.reset();
         socketPair_->stopSendOp(false);
-        sender_.reset(new AudioSender(callID_, getRemoteRtpUri(), send_,
-                                      *socketPair_, initSeqVal_, muteState_, mtu_));
-    } catch (const MediaEncoderException &e) {
+        sender_.reset(
+            new AudioSender(callID_, getRemoteRtpUri(), send_, *socketPair_, initSeqVal_, mtu_));
+    } catch (const MediaEncoderException& e) {
         JAMI_ERR("%s", e.what());
         send_.enabled = false;
     }
+
+    // NOTE do after sender/encoder are ready
+    auto codec = std::static_pointer_cast<AccountAudioCodecInfo>(send_.codec);
+    audioInput_->setFormat(codec->audioformat);
+    if (audioInput_)
+        audioInput_->attach(sender_.get());
+
+    if (not rtcpCheckerThread_.isRunning())
+        rtcpCheckerThread_.start();
 }
 
 void
@@ -121,7 +147,7 @@ AudioRtpSession::restartSender()
 void
 AudioRtpSession::startReceiver()
 {
-    if (not receive_.enabled or receive_.holding) {
+    if (not receive_.enabled or receive_.onHold) {
         JAMI_WARN("Audio receiving disabled");
         receiveThread_.reset();
         return;
@@ -131,11 +157,13 @@ AudioRtpSession::startReceiver()
         JAMI_WARN("Restarting audio receiver");
 
     auto accountAudioCodec = std::static_pointer_cast<AccountAudioCodecInfo>(receive_.codec);
-    receiveThread_.reset(new AudioReceiveThread(callID_, accountAudioCodec->audioformat,
+    receiveThread_.reset(new AudioReceiveThread(callID_,
+                                                accountAudioCodec->audioformat,
                                                 receive_.receiving_sdp,
                                                 mtu_));
     receiveThread_->addIOContext(*socketPair_);
-    receiveThread_->startLoop(onSuccessfulSetup_);
+    receiveThread_->setSuccessfulSetupCb(onSuccessfulSetup_);
+    receiveThread_->startLoop();
 }
 
 void
@@ -150,13 +178,11 @@ AudioRtpSession::start(std::unique_ptr<IceSocket> rtp_sock, std::unique_ptr<IceS
 
     try {
         if (rtp_sock and rtcp_sock) {
-
             rtp_sock->setDefaultRemoteAddress(send_.addr);
             rtcp_sock->setDefaultRemoteAddress(send_.rtcp_addr);
 
             socketPair_.reset(new SocketPair(std::move(rtp_sock), std::move(rtcp_sock)));
-        }
-        else
+        } else
             socketPair_.reset(new SocketPair(getRemoteRtpUri().c_str(), receive_.addr.getPort()));
 
         if (send_.crypto and receive_.crypto) {
@@ -179,21 +205,106 @@ AudioRtpSession::stop()
 {
     std::lock_guard<std::recursive_mutex> lock(mutex_);
 
+    if (audioInput_)
+        audioInput_->detach(sender_.get());
+
     if (socketPair_)
         socketPair_->interrupt();
 
+    rtcpCheckerThread_.join();
+
     receiveThread_.reset();
     sender_.reset();
     socketPair_.reset();
+    audioInput_.reset();
 }
+
 void
 AudioRtpSession::setMuted(bool isMuted)
 {
     std::lock_guard<std::recursive_mutex> lock(mutex_);
-    if (sender_) {
-        muteState_ = isMuted;
-        sender_->setMuted(isMuted);
+    muteState_ = isMuted;
+    if (audioInput_)
+        audioInput_->setMuted(isMuted);
+}
+
+bool
+AudioRtpSession::check_RCTP_Info_RR(RTCPInfo& rtcpi)
+{
+    auto rtcpInfoVect = socketPair_->getRtcpRR();
+    unsigned totalLost = 0;
+    unsigned totalJitter = 0;
+    unsigned nbDropNotNull = 0;
+    auto vectSize = rtcpInfoVect.size();
+
+    if (vectSize != 0) {
+        for (const auto& it : rtcpInfoVect) {
+            if (it.fraction_lost != 0) // Exclude null drop
+                nbDropNotNull++;
+            totalLost += it.fraction_lost;
+            totalJitter += ntohl(it.jitter);
+        }
+        rtcpi.packetLoss = nbDropNotNull ? (float) (100 * totalLost) / (256.0 * nbDropNotNull) : 0;
+        // Jitter is expressed in timestamp unit -> convert to milliseconds
+        // https://stackoverflow.com/questions/51956520/convert-jitter-from-rtp-timestamp-unit-to-millisseconds
+        rtcpi.jitter = (totalJitter / vectSize / 90000.0f) * 1000;
+        rtcpi.nb_sample = vectSize;
+        rtcpi.latency = socketPair_->getLastLatency();
+        return true;
     }
+    return false;
+}
+
+void
+AudioRtpSession::adaptQualityAndBitrate()
+{
+    RTCPInfo rtcpi {};
+    if (check_RCTP_Info_RR(rtcpi)) {
+        dropProcessing(&rtcpi);
+    }
+}
+
+void
+AudioRtpSession::dropProcessing(RTCPInfo* rtcpi)
+{
+    auto pondLoss = getPonderateLoss(rtcpi->packetLoss);
+    setNewPacketLoss(pondLoss);
+}
+
+void
+AudioRtpSession::setNewPacketLoss(unsigned int newPL)
+{
+    newPL = std::clamp((int) newPL, 0, 100);
+    if (newPL != packetLoss_) {
+        if (sender_) {
+            auto ret = sender_->setPacketLoss(newPL);
+            packetLoss_ = newPL;
+            if (ret == -1)
+                JAMI_ERR("Fail to access the encoder");
+        } else {
+            JAMI_ERR("Fail to access the sender");
+        }
+    }
+}
+
+float
+AudioRtpSession::getPonderateLoss(float lastLoss)
+{
+    static float pond = 10.0f;
+
+    pond = floor(0.5 * lastLoss + 0.5 * pond);
+    if (lastLoss > pond) {
+        return lastLoss;
+    } else {
+        return pond;
+    }
+}
+
+void
+AudioRtpSession::processRtcpChecker()
+{
+    adaptQualityAndBitrate();
+    socketPair_->waitForRTCP(std::chrono::seconds(rtcp_checking_interval));
 }
 
 void
