@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2021 Savoir-faire Linux Inc.
  *
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *
@@ -44,7 +44,6 @@ namespace jami {
 OpenSLLayer::OpenSLLayer(const AudioPreference& pref)
     : AudioLayer(pref)
 {
-    initAudioEngine();
 }
 
 // Destructor
@@ -57,8 +56,12 @@ void
 OpenSLLayer::startStream(AudioDeviceType stream)
 {
     using namespace std::placeholders;
+    if (engineObject_)
+        initAudioEngine();
+
     std::lock_guard<std::mutex> lock(mutex_);
     JAMI_WARN("Start OpenSL audio layer");
+
     if (stream == AudioDeviceType::PLAYBACK) {
         if (not player_) {
             try {
@@ -69,6 +72,7 @@ OpenSLLayer::startStream(AudioDeviceType stream)
                 player_->setBufQueue(&playBufQueue_, &freePlayBufQueue_);
                 player_->registerCallback(std::bind(&OpenSLLayer::engineServicePlay, this));
                 player_->start();
+                playbackChanged(true);
             } catch (const std::exception& e) {
                 JAMI_ERR("Error initializing audio playback: %s", e.what());
             }
@@ -93,7 +97,7 @@ OpenSLLayer::startStream(AudioDeviceType stream)
         if (not recorder_) {
             std::lock_guard<std::mutex> lck(recMtx);
             try {
-                recorder_.reset(new opensl::AudioRecorder(hardwareFormat_, engineInterface_));
+                recorder_.reset(new opensl::AudioRecorder(hardwareFormat_, hardwareBuffSize_, engineInterface_));
                 recorder_->setBufQueues(&freeRecBufQueue_, &recBufQueue_);
                 recorder_->registerCallback(std::bind(&OpenSLLayer::engineServiceRec, this));
                 setHasNativeAEC(recorder_->hasNativeAEC());
@@ -111,11 +115,12 @@ OpenSLLayer::startStream(AudioDeviceType stream)
 void
 OpenSLLayer::stopStream(AudioDeviceType stream)
 {
-    std::unique_lock<std::mutex> lock(mutex_);
+    std::lock_guard<std::mutex> lock(mutex_);
     JAMI_WARN("Stopping OpenSL audio layer for type %u", (unsigned) stream);
 
     if (stream == AudioDeviceType::PLAYBACK) {
         if (player_) {
+            playbackChanged(false);
             player_->stop();
             player_.reset();
         }
@@ -150,6 +155,7 @@ allocateSampleBufs(unsigned count, size_t sizeInByte)
 void
 OpenSLLayer::initAudioEngine()
 {
+    JAMI_WARN("OpenSL init started");
     std::vector<int32_t> hw_infos;
     hw_infos.reserve(4);
     emitSignal<DRing::ConfigurationSignal::GetHardwareAudioFormat>(&hw_infos);
@@ -157,6 +163,7 @@ OpenSLLayer::initAudioEngine()
     hardwareBuffSize_ = hw_infos[1];
     hardwareFormatAvailable(hardwareFormat_, hardwareBuffSize_);
 
+    std::lock_guard<std::mutex> lock(mutex_);
     SLASSERT(slCreateEngine(&engineObject_, 0, nullptr, 0, nullptr, nullptr));
     SLASSERT((*engineObject_)->Realize(engineObject_, SL_BOOLEAN_FALSE));
     SLASSERT((*engineObject_)->GetInterface(engineObject_, SL_IID_ENGINE, &engineInterface_));
@@ -169,14 +176,14 @@ OpenSLLayer::initAudioEngine()
         freeRingBufQueue_.push(&bufs_[i]);
     for (int i = 2 * BUF_COUNT; i < 3 * BUF_COUNT; i++)
         freeRecBufQueue_.push(&bufs_[i]);
+    JAMI_WARN("OpenSL init ended");
 }
 
 void
 OpenSLLayer::shutdownAudioEngine()
 {
+    JAMI_DBG("Stopping OpenSL");
     stopAudioCapture();
-    freeRecBufQueue_.clear();
-    recBufQueue_.clear();
 
     if (player_) {
         player_->stop();
@@ -186,10 +193,6 @@ OpenSLLayer::shutdownAudioEngine()
         ringtone_->stop();
         ringtone_.reset();
     }
-    freePlayBufQueue_.clear();
-    playBufQueue_.clear();
-    freeRingBufQueue_.clear();
-    ringBufQueue_.clear();
 
     // destroy engine object, and invalidate all associated interfaces
     JAMI_DBG("Shutdown audio engine");
@@ -199,6 +202,13 @@ OpenSLLayer::shutdownAudioEngine()
         engineInterface_ = nullptr;
     }
 
+    freeRecBufQueue_.clear();
+    recBufQueue_.clear();
+    freePlayBufQueue_.clear();
+    playBufQueue_.clear();
+    freeRingBufQueue_.clear();
+    ringBufQueue_.clear();
+
     startedCv_.notify_all();
     bufs_.clear();
 }
@@ -239,6 +249,15 @@ OpenSLLayer::engineServicePlay()
         if (auto dat = getToPlay(hardwareFormat_, hardwareBuffSize_)) {
             buf->size_ = dat->pointer()->nb_samples * dat->pointer()->channels
                          * sizeof(AudioSample);
+            if (buf->size_ > buf->cap_) {
+                JAMI_ERR("buf->size_(%zu) > buf->cap_(%zu)", buf->size_, buf->cap_);
+                break;
+            }
+            if (not dat->pointer()->data[0] or not buf->buf_) {
+                JAMI_ERR("null bufer %p -> %p %d", dat->pointer()->data[0], buf->buf_, dat->pointer()->nb_samples);
+                break;
+            }
+            //JAMI_ERR("std::copy_n %p -> %p %zu", dat->pointer()->data[0], buf->buf_, dat->pointer()->nb_samples);
             std::copy_n((const AudioSample*) dat->pointer()->data[0],
                         dat->pointer()->nb_samples,
                         (AudioSample*) buf->buf_);
@@ -262,6 +281,14 @@ OpenSLLayer::engineServiceRing()
         if (auto dat = getToRing(hardwareFormat_, hardwareBuffSize_)) {
             buf->size_ = dat->pointer()->nb_samples * dat->pointer()->channels
                          * sizeof(AudioSample);
+            if (buf->size_ > buf->cap_) {
+                JAMI_ERR("buf->size_(%zu) > buf->cap_(%zu)", buf->size_, buf->cap_);
+                break;
+            }
+            if (not dat->pointer()->data[0] or not buf->buf_) {
+                JAMI_ERR("null bufer %p -> %p %d", dat->pointer()->data[0], buf->buf_, dat->pointer()->nb_samples);
+                break;
+            }
             std::copy_n((const AudioSample*) dat->pointer()->data[0],
                         dat->pointer()->nb_samples,
                         (AudioSample*) buf->buf_);
@@ -284,21 +311,6 @@ OpenSLLayer::engineServiceRec()
     return;
 }
 
-void
-OpenSLLayer::initAudioCapture()
-{
-    using namespace std::placeholders;
-    std::lock_guard<std::mutex> lck(recMtx);
-    try {
-        recorder_.reset(new opensl::AudioRecorder(hardwareFormat_, engineInterface_));
-        recorder_->setBufQueues(&freeRecBufQueue_, &recBufQueue_);
-        recorder_->registerCallback(std::bind(&OpenSLLayer::engineServiceRec, this));
-        setHasNativeAEC(recorder_->hasNativeAEC());
-    } catch (const std::exception& e) {
-        JAMI_ERR("Error initializing audio capture: %s", e.what());
-    }
-}
-
 void
 OpenSLLayer::startAudioCapture()
 {
