@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2021 Savoir-faire Linux Inc.
  *
  *  Author: Alexandre Savard <alexandre.savard@savoirfairelinux.com>
  *  Author: Philippe Gorley <philippe.gorley@savoirfairelinux.com>
@@ -64,7 +64,6 @@
 #pragma GCC diagnostic pop
 
 #include "config/yamlparser.h"
-#include "hooks/urlhook.h"
 #include "sip/sip_utils.h"
 #include <sstream>
 #include <algorithm>
@@ -76,9 +75,10 @@ namespace jami {
 
 using yaml_utils::parseValue;
 
-constexpr const char * const Preferences::CONFIG_LABEL;
-const char * const Preferences::DFT_ZONE = "North America";
-const char * const Preferences::REGISTRATION_EXPIRE_KEY = "registrationexpire";
+constexpr const char* const Preferences::CONFIG_LABEL;
+const char* const Preferences::DFT_ZONE = "North America";
+const char* const Preferences::REGISTRATION_EXPIRE_KEY = "registrationexpire";
+constexpr std::string_view DEFAULT_CONFERENCE_RESOLUTION {"1280x720"};
 
 // general preferences
 static constexpr const char* ORDER_KEY {"order"};
@@ -92,7 +92,7 @@ static constexpr const char* SEARCH_BAR_DISPLAY_KEY {"searchBarDisplay"};
 static constexpr const char* MD5_HASH_KEY {"md5Hash"};
 
 // voip preferences
-constexpr const char * const VoipPreference::CONFIG_LABEL;
+constexpr const char* const VoipPreference::CONFIG_LABEL;
 static constexpr const char* DISABLE_SECURE_DLG_CHECK_KEY {"disableSecureDlgCheck"};
 static constexpr const char* PLAY_DTMF_KEY {"playDtmf"};
 static constexpr const char* PLAY_TONES_KEY {"playTones"};
@@ -100,18 +100,11 @@ static constexpr const char* PULSE_LENGTH_KEY {"pulseLength"};
 static constexpr const char* SYMMETRIC_RTP_KEY {"symmetric"};
 static constexpr const char* ZID_FILE_KEY {"zidFile"};
 
-// hooks preferences
-constexpr const char * const HookPreference::CONFIG_LABEL;
-static constexpr const char* NUMBER_ADD_PREFIX_KEY {"numberAddPrefix"};
-static constexpr const char* NUMBER_ENABLED_KEY {"numberEnabled"};
-static constexpr const char* SIP_ENABLED_KEY {"sipEnabled"};
-static constexpr const char* URL_COMMAND_KEY {"urlCommand"};
-static constexpr const char* URL_SIP_FIELD_KEY {"urlSipField"};
-
 // audio preferences
-constexpr const char * const AudioPreference::CONFIG_LABEL;
+constexpr const char* const AudioPreference::CONFIG_LABEL;
 static constexpr const char* ALSAMAP_KEY {"alsa"};
 static constexpr const char* PULSEMAP_KEY {"pulse"};
+static constexpr const char* PORTAUDIO_KEY {"portaudio"};
 static constexpr const char* CARDIN_KEY {"cardIn"};
 static constexpr const char* CARDOUT_KEY {"cardOut"};
 static constexpr const char* CARDRING_KEY {"cardRing"};
@@ -131,7 +124,7 @@ static constexpr const char* CAPTURE_MUTED_KEY {"captureMuted"};
 static constexpr const char* PLAYBACK_MUTED_KEY {"playbackMuted"};
 
 // shortcut preferences
-constexpr const char * const ShortcutPreferences::CONFIG_LABEL;
+constexpr const char* const ShortcutPreferences::CONFIG_LABEL;
 static constexpr const char* HANGUP_SHORT_KEY {"hangUp"};
 static constexpr const char* PICKUP_SHORT_KEY {"pickUp"};
 static constexpr const char* POPUP_SHORT_KEY {"popupWindow"};
@@ -140,30 +133,31 @@ static constexpr const char* TOGGLE_PICKUP_HANGUP_SHORT_KEY {"togglePickupHangup
 
 #ifdef ENABLE_VIDEO
 // video preferences
-constexpr const char * const VideoPreferences::CONFIG_LABEL;
+constexpr const char* const VideoPreferences::CONFIG_LABEL;
 static constexpr const char* DECODING_ACCELERATED_KEY {"decodingAccelerated"};
 static constexpr const char* ENCODING_ACCELERATED_KEY {"encodingAccelerated"};
 static constexpr const char* RECORD_PREVIEW_KEY {"recordPreview"};
 static constexpr const char* RECORD_QUALITY_KEY {"recordQuality"};
+static constexpr const char* CONFERENCE_RESOLUTION_KEY {"conferenceResolution"};
 #endif
 
 #ifdef ENABLE_PLUGIN
 // plugin preferences
-constexpr const char * const PluginPreferences::CONFIG_LABEL;
+constexpr const char* const PluginPreferences::CONFIG_LABEL;
 static constexpr const char* JAMI_PLUGIN_KEY {"pluginsEnabled"};
+static constexpr const char* JAMI_PLUGINS_INSTALLED_KEY {"installedPlugins"};
 static constexpr const char* JAMI_PLUGINS_LOADED_KEY {"loadedPlugins"};
 #endif
 
 static constexpr int PULSE_LENGTH_DEFAULT {250}; /** Default DTMF length */
 #ifndef _MSC_VER
-static constexpr const char* ALSA_DFT_CARD {"0"};          /** Default sound card index */
+static constexpr const char* ALSA_DFT_CARD {"0"}; /** Default sound card index */
 #else
-static constexpr const char* ALSA_DFT_CARD {"-1"};         /** Default sound card index (Portaudio) */
+static constexpr const char* ALSA_DFT_CARD {"-1"}; /** Default sound card index (Portaudio) */
 #endif // _MSC_VER
 
-
-Preferences::Preferences() :
-    accountOrder_("")
+Preferences::Preferences()
+    : accountOrder_("")
     , historyLimit_(0)
     , historyMaxCalls_(20)
     , ringingTimeout_(30)
@@ -174,7 +168,8 @@ Preferences::Preferences() :
     , md5Hash_(false)
 {}
 
-void Preferences::verifyAccountOrder(const std::vector<std::string> &accountIDs)
+void
+Preferences::verifyAccountOrder(const std::vector<std::string>& accountIDs)
 {
     std::vector<std::string> tokens;
     std::string token;
@@ -196,12 +191,13 @@ void Preferences::verifyAccountOrder(const std::vector<std::string> &accountIDs)
 
     if (drop) {
         accountOrder_.clear();
-        for (const auto &t : tokens)
-            accountOrder_ += t + "/";
+        for (const auto& t : tokens)
+            accountOrder_ += t + '/';
     }
 }
 
-void Preferences::addAccount(const std::string &newAccountID)
+void
+Preferences::addAccount(const std::string& newAccountID)
 {
     // Add the newly created account in the account order list
     if (not accountOrder_.empty())
@@ -210,7 +206,8 @@ void Preferences::addAccount(const std::string &newAccountID)
         accountOrder_ = newAccountID + "/";
 }
 
-void Preferences::removeAccount(const std::string &oldAccountID)
+void
+Preferences::removeAccount(const std::string& oldAccountID)
 {
     // include the slash since we don't want to remove a partial match
     const size_t start = accountOrder_.find(oldAccountID + "/");
@@ -218,7 +215,8 @@ void Preferences::removeAccount(const std::string &oldAccountID)
         accountOrder_.erase(start, oldAccountID.length() + 1);
 }
 
-void Preferences::serialize(YAML::Emitter &out) const
+void
+Preferences::serialize(YAML::Emitter& out) const
 {
     out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
 
@@ -234,9 +232,10 @@ void Preferences::serialize(YAML::Emitter &out) const
     out << YAML::EndMap;
 }
 
-void Preferences::unserialize(const YAML::Node &in)
+void
+Preferences::unserialize(const YAML::Node& in)
 {
-    const auto &node = in[CONFIG_LABEL];
+    const auto& node = in[CONFIG_LABEL];
 
     parseValue(node, ORDER_KEY, accountOrder_);
     parseValue(node, HISTORY_LIMIT_KEY, historyLimit_);
@@ -249,15 +248,16 @@ void Preferences::unserialize(const YAML::Node &in)
     parseValue(node, MD5_HASH_KEY, md5Hash_);
 }
 
-VoipPreference::VoipPreference() :
-    disableSecureDlgCheck_(false)
+VoipPreference::VoipPreference()
+    : disableSecureDlgCheck_(false)
     , playDtmf_(true)
     , playTones_(true)
     , pulseLength_(PULSE_LENGTH_DEFAULT)
     , symmetricRtp_(true)
 {}
 
-void VoipPreference::serialize(YAML::Emitter &out) const
+void
+VoipPreference::serialize(YAML::Emitter& out) const
 {
     out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
     out << YAML::Key << DISABLE_SECURE_DLG_CHECK_KEY << YAML::Value << disableSecureDlgCheck_;
@@ -269,9 +269,10 @@ void VoipPreference::serialize(YAML::Emitter &out) const
     out << YAML::EndMap;
 }
 
-void VoipPreference::unserialize(const YAML::Node &in)
+void
+VoipPreference::unserialize(const YAML::Node& in)
 {
-    const auto &node = in[CONFIG_LABEL];
+    const auto& node = in[CONFIG_LABEL];
     parseValue(node, DISABLE_SECURE_DLG_CHECK_KEY, disableSecureDlgCheck_);
     parseValue(node, PLAY_DTMF_KEY, playDtmf_);
     parseValue(node, PLAY_TONES_KEY, playTones_);
@@ -280,64 +281,8 @@ void VoipPreference::unserialize(const YAML::Node &in)
     parseValue(node, ZID_FILE_KEY, zidFile_);
 }
 
-HookPreference::HookPreference()
-    : numberAddPrefix_("")
-    , numberEnabled_(false)
-    , sipEnabled_(false)
-    , urlCommand_("x-www-browser")
-    , urlSipField_("X-ring-url")
-{}
-
-HookPreference::HookPreference(const std::map<std::string, std::string> &settings)
-    : numberAddPrefix_(settings.find("PHONE_NUMBER_HOOK_ADD_PREFIX")->second)
-    , numberEnabled_(settings.find("PHONE_NUMBER_HOOK_ENABLED")->second == "true")
-    , sipEnabled_(settings.find("URLHOOK_SIP_ENABLED")->second == "true")
-    , urlCommand_(settings.find("URLHOOK_COMMAND")->second)
-    , urlSipField_(settings.find("URLHOOK_SIP_FIELD")->second)
-{}
-
-std::map<std::string, std::string> HookPreference::toMap() const
-{
-    std::map<std::string, std::string> settings;
-    settings["PHONE_NUMBER_HOOK_ADD_PREFIX"] = numberAddPrefix_;
-    settings["PHONE_NUMBER_HOOK_ENABLED"] = numberEnabled_ ? "true" : "false";
-    settings["URLHOOK_SIP_ENABLED"] = sipEnabled_ ? "true" : "false";
-    settings["URLHOOK_COMMAND"] = urlCommand_;
-    settings["URLHOOK_SIP_FIELD"] = urlSipField_;
-
-    return settings;
-}
-
-void HookPreference::serialize(YAML::Emitter &out) const
-{
-    out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
-    out << YAML::Key << NUMBER_ADD_PREFIX_KEY << YAML::Value << numberAddPrefix_;
-    out << YAML::Key << SIP_ENABLED_KEY << YAML::Value << sipEnabled_;
-    out << YAML::Key << URL_COMMAND_KEY << YAML::Value << urlCommand_;
-    out << YAML::Key << URL_SIP_FIELD_KEY << YAML::Value << urlSipField_;
-    out << YAML::EndMap;
-}
-
-void HookPreference::unserialize(const YAML::Node &in)
-{
-    const auto &node = in[CONFIG_LABEL];
-
-    parseValue(node, NUMBER_ADD_PREFIX_KEY, numberAddPrefix_);
-    parseValue(node, SIP_ENABLED_KEY, sipEnabled_);
-    parseValue(node, URL_COMMAND_KEY, urlCommand_);
-    parseValue(node, URL_SIP_FIELD_KEY, urlSipField_);
-}
-
-void HookPreference::runHook(pjsip_msg *msg)
-{
-    if (sipEnabled_) {
-        const std::string header(sip_utils::fetchHeaderValue(msg, urlSipField_));
-        UrlHook::runAction(urlCommand_, header);
-    }
-}
-
-AudioPreference::AudioPreference() :
-    audioApi_(PULSEAUDIO_API_STR)
+AudioPreference::AudioPreference()
+    : audioApi_(PULSEAUDIO_API_STR)
     , alsaCardin_(atoi(ALSA_DFT_CARD))
     , alsaCardout_(atoi(ALSA_DFT_CARD))
     , alsaCardring_(atoi(ALSA_DFT_CARD))
@@ -362,7 +307,7 @@ AudioPreference::AudioPreference() :
 static const int ALSA_DFT_CARD_ID = 0; // Index of the default soundcard
 
 static void
-checkSoundCard(int &card, DeviceType type)
+checkSoundCard(int& card, AudioDeviceType type)
 {
     if (not AlsaLayer::soundCardIndexExists(card, type)) {
         JAMI_WARN(" Card with index %d doesn't exist or is unusable.", card);
@@ -406,7 +351,7 @@ AudioPreference::createAudioLayer()
     if (audioApi_ == PULSEAUDIO_API_STR) {
         try {
             return new PulseLayer(*this);
-        } catch (const std::runtime_error &e) {
+        } catch (const std::runtime_error& e) {
             JAMI_WARN("Could not create pulseaudio layer, falling back to ALSA");
         }
     }
@@ -416,9 +361,9 @@ AudioPreference::createAudioLayer()
 #if HAVE_ALSA
 
     audioApi_ = ALSA_API_STR;
-    checkSoundCard(alsaCardin_, DeviceType::CAPTURE);
-    checkSoundCard(alsaCardout_, DeviceType::PLAYBACK);
-    checkSoundCard(alsaCardring_, DeviceType::RINGTONE);
+    checkSoundCard(alsaCardin_, AudioDeviceType::CAPTURE);
+    checkSoundCard(alsaCardout_, AudioDeviceType::PLAYBACK);
+    checkSoundCard(alsaCardring_, AudioDeviceType::RINGTONE);
 
     return new AlsaLayer(*this);
 #endif
@@ -427,7 +372,7 @@ AudioPreference::createAudioLayer()
     audioApi_ = COREAUDIO_API_STR;
     try {
         return new CoreLayer(*this);
-    } catch (const std::runtime_error &e) {
+    } catch (const std::runtime_error& e) {
         JAMI_WARN("Could not create coreaudio layer. There will be no sound.");
     }
     return NULL;
@@ -437,7 +382,7 @@ AudioPreference::createAudioLayer()
     audioApi_ = PORTAUDIO_API_STR;
     try {
         return new PortAudioLayer(*this);
-    } catch (const std::runtime_error &e) {
+    } catch (const std::runtime_error& e) {
         JAMI_WARN("Could not create PortAudio layer. There will be no sound.");
     }
     return nullptr;
@@ -451,29 +396,31 @@ AudioPreference::createAudioLayer()
 std::vector<std::string>
 AudioPreference::getSupportedAudioManagers()
 {
-    return {
+    return
+    {
 #if HAVE_OPENSL
         OPENSL_API_STR,
 #endif
 #if HAVE_ALSA
-        ALSA_API_STR,
+            ALSA_API_STR,
 #endif
 #if HAVE_PULSE
-        PULSEAUDIO_API_STR,
+            PULSEAUDIO_API_STR,
 #endif
 #if HAVE_JACK
-        JACK_API_STR,
+            JACK_API_STR,
 #endif
 #if HAVE_COREAUDIO
-        COREAUDIO_API_STR,
+            COREAUDIO_API_STR,
 #endif
 #if HAVE_PORTAUDIO
-        PORTAUDIO_API_STR,
+            PORTAUDIO_API_STR,
 #endif
     };
 }
 
-void AudioPreference::serialize(YAML::Emitter &out) const
+void
+AudioPreference::serialize(YAML::Emitter& out) const
 {
     out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
     // alsa submap
@@ -500,6 +447,13 @@ void AudioPreference::serialize(YAML::Emitter &out) const
     out << YAML::Key << DEVICE_RINGTONE_KEY << YAML::Value << pulseDeviceRingtone_;
     out << YAML::EndMap;
 
+    // portaudio submap
+    out << YAML::Key << PORTAUDIO_KEY << YAML::Value << YAML::BeginMap;
+    out << YAML::Key << DEVICE_PLAYBACK_KEY << YAML::Value << portaudioDevicePlayback_;
+    out << YAML::Key << DEVICE_RECORD_KEY << YAML::Value << portaudioDeviceRecord_;
+    out << YAML::Key << DEVICE_RINGTONE_KEY << YAML::Value << portaudioDeviceRingtone_;
+    out << YAML::EndMap;
+
     // more common options!
     out << YAML::Key << RECORDPATH_KEY << YAML::Value << recordpath_;
     out << YAML::Key << VOLUMEMIC_KEY << YAML::Value << volumemic_;
@@ -509,7 +463,7 @@ void AudioPreference::serialize(YAML::Emitter &out) const
 }
 
 bool
-AudioPreference::setRecordPath(const std::string &r)
+AudioPreference::setRecordPath(const std::string& r)
 {
     std::string path = fileutils::expand_path(r);
     if (fileutils::isDirectoryWritable(path)) {
@@ -521,12 +475,13 @@ AudioPreference::setRecordPath(const std::string &r)
     }
 }
 
-void AudioPreference::unserialize(const YAML::Node &in)
+void
+AudioPreference::unserialize(const YAML::Node& in)
 {
-    const auto &node = in[CONFIG_LABEL];
+    const auto& node = in[CONFIG_LABEL];
 
     // alsa submap
-    const auto &alsa = node[ALSAMAP_KEY];
+    const auto& alsa = node[ALSAMAP_KEY];
 
     parseValue(alsa, CARDIN_KEY, alsaCardin_);
     parseValue(alsa, CARDOUT_KEY, alsaCardout_);
@@ -543,11 +498,17 @@ void AudioPreference::unserialize(const YAML::Node &in)
     parseValue(node, PLAYBACK_MUTED_KEY, playbackMuted_);
 
     // pulse submap
-    const auto &pulse = node[PULSEMAP_KEY];
+    const auto& pulse = node[PULSEMAP_KEY];
     parseValue(pulse, DEVICE_PLAYBACK_KEY, pulseDevicePlayback_);
     parseValue(pulse, DEVICE_RECORD_KEY, pulseDeviceRecord_);
     parseValue(pulse, DEVICE_RINGTONE_KEY, pulseDeviceRingtone_);
 
+    // portaudio submap
+    const auto& portaudio = node[PORTAUDIO_KEY];
+    parseValue(portaudio, DEVICE_PLAYBACK_KEY, portaudioDevicePlayback_);
+    parseValue(portaudio, DEVICE_RECORD_KEY, portaudioDeviceRecord_);
+    parseValue(portaudio, DEVICE_RINGTONE_KEY, portaudioDeviceRingtone_);
+
     // more common options!
     parseValue(node, RECORDPATH_KEY, recordpath_);
     parseValue(node, VOLUMEMIC_KEY, volumemic_);
@@ -555,10 +516,16 @@ void AudioPreference::unserialize(const YAML::Node &in)
     parseValue(node, ECHO_CANCELLER, echoCanceller_);
 }
 
-ShortcutPreferences::ShortcutPreferences() : hangup_(), pickup_(), popup_(),
-    toggleHold_(), togglePickupHangup_() {}
+ShortcutPreferences::ShortcutPreferences()
+    : hangup_()
+    , pickup_()
+    , popup_()
+    , toggleHold_()
+    , togglePickupHangup_()
+{}
 
-std::map<std::string, std::string> ShortcutPreferences::getShortcuts() const
+std::map<std::string, std::string>
+ShortcutPreferences::getShortcuts() const
 {
     std::map<std::string, std::string> shortcutsMap;
 
@@ -571,7 +538,8 @@ std::map<std::string, std::string> ShortcutPreferences::getShortcuts() const
     return shortcutsMap;
 }
 
-void ShortcutPreferences::setShortcuts(std::map<std::string, std::string> map)
+void
+ShortcutPreferences::setShortcuts(std::map<std::string, std::string> map)
 {
     hangup_ = map[HANGUP_SHORT_KEY];
     pickup_ = map[PICKUP_SHORT_KEY];
@@ -580,8 +548,8 @@ void ShortcutPreferences::setShortcuts(std::map<std::string, std::string> map)
     togglePickupHangup_ = map[TOGGLE_PICKUP_HANGUP_SHORT_KEY];
 }
 
-
-void ShortcutPreferences::serialize(YAML::Emitter &out) const
+void
+ShortcutPreferences::serialize(YAML::Emitter& out) const
 {
     out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
     out << YAML::Key << HANGUP_SHORT_KEY << YAML::Value << hangup_;
@@ -592,9 +560,10 @@ void ShortcutPreferences::serialize(YAML::Emitter &out) const
     out << YAML::EndMap;
 }
 
-void ShortcutPreferences::unserialize(const YAML::Node &in)
+void
+ShortcutPreferences::unserialize(const YAML::Node& in)
 {
-    const auto &node = in[CONFIG_LABEL];
+    const auto& node = in[CONFIG_LABEL];
 
     parseValue(node, HANGUP_SHORT_KEY, hangup_);
     parseValue(node, PICKUP_SHORT_KEY, pickup_);
@@ -609,10 +578,11 @@ VideoPreferences::VideoPreferences()
     , encodingAccelerated_(false)
     , recordPreview_(true)
     , recordQuality_(0)
-{
-}
+    , conferenceResolution_(DEFAULT_CONFERENCE_RESOLUTION)
+{}
 
-void VideoPreferences::serialize(YAML::Emitter &out) const
+void
+VideoPreferences::serialize(YAML::Emitter& out) const
 {
     out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
     out << YAML::Key << RECORD_PREVIEW_KEY << YAML::Value << recordPreview_;
@@ -621,14 +591,16 @@ void VideoPreferences::serialize(YAML::Emitter &out) const
     out << YAML::Key << DECODING_ACCELERATED_KEY << YAML::Value << decodingAccelerated_;
     out << YAML::Key << ENCODING_ACCELERATED_KEY << YAML::Value << encodingAccelerated_;
 #endif
+    out << YAML::Key << CONFERENCE_RESOLUTION_KEY << YAML::Value << conferenceResolution_;
     getVideoDeviceMonitor().serialize(out);
     out << YAML::EndMap;
 }
 
-void VideoPreferences::unserialize(const YAML::Node &in)
+void
+VideoPreferences::unserialize(const YAML::Node& in)
 {
     // values may or may not be present
-    const auto &node = in[CONFIG_LABEL];
+    const auto& node = in[CONFIG_LABEL];
     try {
         parseValue(node, RECORD_PREVIEW_KEY, recordPreview_);
         parseValue(node, RECORD_QUALITY_KEY, recordQuality_);
@@ -645,6 +617,11 @@ void VideoPreferences::unserialize(const YAML::Node &in)
         encodingAccelerated_ = false;
     }
 #endif
+    try {
+        parseValue(node, CONFERENCE_RESOLUTION_KEY, conferenceResolution_);
+    } catch (...) {
+        conferenceResolution_ = DEFAULT_CONFERENCE_RESOLUTION;
+    }
     getVideoDeviceMonitor().unserialize(in);
 }
 #endif // ENABLE_VIDEO
@@ -654,25 +631,34 @@ PluginPreferences::PluginPreferences()
     : pluginsEnabled_(false)
 {}
 
-void PluginPreferences::serialize(YAML::Emitter &out) const
+void
+PluginPreferences::serialize(YAML::Emitter& out) const
 {
     out << YAML::Key << CONFIG_LABEL << YAML::Value << YAML::BeginMap;
     out << YAML::Key << JAMI_PLUGIN_KEY << YAML::Value << pluginsEnabled_;
+    out << YAML::Key << JAMI_PLUGINS_INSTALLED_KEY << YAML::Value << installedPlugins_;
     out << YAML::Key << JAMI_PLUGINS_LOADED_KEY << YAML::Value << loadedPlugins_;
     out << YAML::EndMap;
 }
 
-void PluginPreferences::unserialize(const YAML::Node &in)
+void
+PluginPreferences::unserialize(const YAML::Node& in)
 {
     // values may or may not be present
-    const auto &node = in[CONFIG_LABEL];
+    const auto& node = in[CONFIG_LABEL];
     try {
         parseValue(node, JAMI_PLUGIN_KEY, pluginsEnabled_);
     } catch (...) {
         pluginsEnabled_ = false;
     }
 
-    const auto &loadedPluginsNode = node[JAMI_PLUGINS_LOADED_KEY];
+    const auto& installedPluginsNode = node[JAMI_PLUGINS_INSTALLED_KEY];
+    try {
+        installedPlugins_ = yaml_utils::parseVector(installedPluginsNode);
+    } catch (...) {
+    }
+
+    const auto& loadedPluginsNode = node[JAMI_PLUGINS_LOADED_KEY];
     try {
         loadedPlugins_ = yaml_utils::parseVector(loadedPluginsNode);
     } catch (...) {
