@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2021 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
@@ -32,7 +32,6 @@
 #include "manager.h"
 #include "certstore.h"
 #include "array_size.h"
-#include "diffie-hellman.h"
 #include "scheduled_executor.h"
 
 #include <gnutls/gnutls.h>
@@ -108,8 +107,10 @@ static constexpr auto RX_OOO_TIMEOUT = std::chrono::milliseconds(1500);
 static constexpr int ASYMETRIC_TRANSPORT_MTU_OFFSET
     = 20; // when client, if your local IP is IPV4 and server is IPV6; you must reduce your MTU to
           // avoid packet too big error on server side. the offset is the difference in size of IP headers
-static constexpr auto OCSP_VERIFICATION_TIMEOUT = std::chrono::seconds(2); // Time to wait for an OCSP verification
-static constexpr auto OCSP_REQUEST_TIMEOUT = std::chrono::seconds(2); // Time to wait for an ocsp-request
+static constexpr auto OCSP_VERIFICATION_TIMEOUT = std::chrono::seconds(
+    2); // Time to wait for an OCSP verification
+static constexpr auto OCSP_REQUEST_TIMEOUT = std::chrono::seconds(
+    2); // Time to wait for an ocsp-request
 
 // Helper to cast any duration into an integer number of milliseconds
 template<class Rep, class Period>
@@ -281,7 +282,8 @@ public:
     std::unique_ptr<TlsAnonymousClientCredendials> cacred_; // ctor init.
     std::unique_ptr<TlsAnonymousServerCredendials> sacred_; // ctor init.
     std::unique_ptr<TlsCertificateCredendials> xcred_;      // ctor init.
-    std::mutex sessionMutex_;
+    std::mutex sessionReadMutex_;
+    std::mutex sessionWriteMutex_;
     gnutls_session_t session_ {nullptr};
     gnutls_datum_t cookie_key_ {nullptr, 0};
     gnutls_dtls_prestate_st prestate_ {};
@@ -300,12 +302,17 @@ public:
     /*
      * Verify OCSP (Online Certificate Service Protocol):
      */
-    void verifyOcsp(const std::string& url, dht::crypto::Certificate& cert, gnutls_x509_crt_t issuer, OcspVerification cb);
+    void verifyOcsp(const std::string& url,
+                    dht::crypto::Certificate& cert,
+                    gnutls_x509_crt_t issuer,
+                    OcspVerification cb);
     /*
      * Send OCSP Request to the specified URI.
      */
-    void sendOcspRequest(const std::string& uri, std::string body,
-                         std::chrono::seconds timeout, HttpResponse cb = {});
+    void sendOcspRequest(const std::string& uri,
+                         std::string body,
+                         std::chrono::seconds timeout,
+                         HttpResponse cb = {});
 
     // FSM thread (TLS states)
     ThreadLoop thread_; // ctor init.
@@ -610,7 +617,9 @@ TlsSession::TlsSessionImpl::commonSessionInit()
                                                    return this_->waitForRawData(
                                                        std::chrono::milliseconds(ms));
                                                });
-
+    // TODO -1 = default else set value
+    if (transport_->isReliable())
+        gnutls_handshake_set_timeout(session_, duration2ms(params_.timeout));
     return true;
 }
 
@@ -623,13 +632,12 @@ getOcspUrl(gnutls_x509_crt_t cert)
     do {
         // Extracts the Authority Information Access (AIA) extension, see RFC 5280 section 4.2.2.1
         ret = gnutls_x509_crt_get_authority_info_access(cert, seq++, GNUTLS_IA_OCSP_URI, &aia, NULL);
-    }
-    while (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
+    } while (ret < 0 && ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
     // could also try the issuer if we include ocsp uri into there
     if (ret < 0) {
         return {};
     }
-    std::string url((const char*)aia.data,(size_t)aia.size);
+    std::string url((const char*) aia.data, (size_t) aia.size);
     gnutls_free(aia.data);
     return url;
 }
@@ -639,12 +647,13 @@ TlsSession::TlsSessionImpl::verifyCertificateWrapper(gnutls_session_t session)
 {
     // Perform user-set verification first to avoid flooding with ocsp-requests if peer is denied
     int verified;
-    if (callbacks_.verifyCertificate)
-    {
+    if (callbacks_.verifyCertificate) {
         auto this_ = reinterpret_cast<TlsSessionImpl*>(gnutls_session_get_ptr(session));
         verified = this_->callbacks_.verifyCertificate(session);
         if (verified != GNUTLS_E_SUCCESS)
             return verified;
+    } else {
+        verified = GNUTLS_E_SUCCESS;
     }
     /*
      * Support only x509 format
@@ -671,7 +680,7 @@ TlsSession::TlsSessionImpl::verifyCertificateWrapper(gnutls_session_t session)
 
     std::string ocspUrl = getOcspUrl(cert.cert);
     if (ocspUrl.empty()) {
-        JAMI_DBG("Skipping OCSP verification %s: AIA not found", cert.getUID().c_str());
+        // Skipping OCSP verification: AIA not found
         return verified;
     }
 
@@ -680,16 +689,18 @@ TlsSession::TlsSessionImpl::verifyCertificateWrapper(gnutls_session_t session)
     std::mutex cv_m;
     std::condition_variable cv;
     std::unique_lock<std::mutex> cv_lk(cv_m);
-    
+
     gnutls_x509_crt_t issuer_crt = cert.issuer ? cert.issuer->cert : nullptr;
     verifyOcsp(ocspUrl, cert, issuer_crt, [&](const int status) {
-        if (status == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE){
+        if (status == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
             // OCSP URI is absent, don't fail the verification by overwritting the user-set one.
             JAMI_WARN("Skipping OCSP verification %s: request failed", cert.getUID().c_str());
         } else {
             if (status != GNUTLS_E_SUCCESS) {
                 JAMI_ERR("OCSP verification failed for %s: %s (%i)",
-                     cert.getUID().c_str(), gnutls_strerror(status), status);
+                         cert.getUID().c_str(),
+                         gnutls_strerror(status),
+                         status);
             }
             verified = status;
         }
@@ -697,14 +708,19 @@ TlsSession::TlsSessionImpl::verifyCertificateWrapper(gnutls_session_t session)
         ocsp_done = true;
         cv.notify_all();
     });
-    cv.wait_for(cv_lk, std::chrono::seconds(OCSP_VERIFICATION_TIMEOUT),[&ocsp_done]{return ocsp_done;});
+    cv.wait_for(cv_lk, std::chrono::seconds(OCSP_VERIFICATION_TIMEOUT), [&ocsp_done] {
+        return ocsp_done;
+    });
     // } OCSP
 
     return verified;
 }
 
 void
-TlsSession::TlsSessionImpl::verifyOcsp(const std::string& aia_uri, dht::crypto::Certificate& cert, gnutls_x509_crt_t issuer, OcspVerification cb)
+TlsSession::TlsSessionImpl::verifyOcsp(const std::string& aia_uri,
+                                       dht::crypto::Certificate& cert,
+                                       gnutls_x509_crt_t issuer,
+                                       OcspVerification cb)
 {
     JAMI_DBG("Certificate's AIA URI: %s", aia_uri.c_str());
 
@@ -712,95 +728,94 @@ TlsSession::TlsSessionImpl::verifyOcsp(const std::string& aia_uri, dht::crypto::
     std::pair<std::string, dht::Blob> ocsp_req;
     try {
         ocsp_req = cert.generateOcspRequest(issuer);
-    }
-    catch (dht::crypto::CryptoException& e){
+    } catch (dht::crypto::CryptoException& e) {
         JAMI_ERR("Failed to generate OCSP request: %s", e.what());
         if (cb)
             cb(GNUTLS_E_INVALID_REQUEST);
         return;
     }
 
-    sendOcspRequest(aia_uri, std::move(ocsp_req.first), OCSP_REQUEST_TIMEOUT,
-        [this, cb = std::move(cb), &cert, nonce=std::move(ocsp_req.second)](const dht::http::Response& r){
-        // Prepare response data
-        // Verify response validity
-        if (r.status_code != 200) {
-            JAMI_WARN("HTTP OCSP Request Failed with code %i", r.status_code);
-            if (cb)
-                cb(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
-            return;
-        }
-        JAMI_DBG("HTTP OCSP Request done!");
-        unsigned int verify = 0;
-        try {
-            cert.ocspResponse = std::make_shared<dht::crypto::OcspResponse>((const uint8_t*)r.body.data(), r.body.size());
-            JAMI_DBG("%s", cert.ocspResponse->toString().c_str());
-            verify = cert.ocspResponse->verifyDirect(cert, nonce);
-        }
-        catch (dht::crypto::CryptoException& e) {
-            JAMI_ERR("Failed to verify OCSP response: %s", e.what());
-            if (cb)
-                cb(GNUTLS_E_INVALID_REQUEST);
-            return;
-        }
-        if (verify == 0)
-            JAMI_DBG("OCSP verification success!");
-        else
-            JAMI_ERR("OCSP verification error!");
-        if (verify & GNUTLS_OCSP_VERIFY_SIGNER_NOT_FOUND)
-            JAMI_ERR("Signer cert not found");
-        if (verify & GNUTLS_OCSP_VERIFY_SIGNER_KEYUSAGE_ERROR)
-            JAMI_ERR("Signer cert keyusage error");
-        if (verify & GNUTLS_OCSP_VERIFY_UNTRUSTED_SIGNER)
-            JAMI_ERR("Signer cert is not trusted");
-        if (verify & GNUTLS_OCSP_VERIFY_INSECURE_ALGORITHM)
-            JAMI_ERR("Insecure algorithm");
-        if (verify & GNUTLS_OCSP_VERIFY_SIGNATURE_FAILURE)
-            JAMI_ERR("Signature failure");
-        if (verify & GNUTLS_OCSP_VERIFY_CERT_NOT_ACTIVATED)
-            JAMI_ERR("Signer cert not yet activated");
-        if (verify & GNUTLS_OCSP_VERIFY_CERT_EXPIRED)
-            JAMI_ERR("Signer cert expired");
-        // Save response into the certificate store
-        tls::CertificateStore::instance().pinOcspResponse(cert);
-        if (cb)
-            cb(verify);
-    });
+    sendOcspRequest(aia_uri,
+                    std::move(ocsp_req.first),
+                    OCSP_REQUEST_TIMEOUT,
+                    [this, cb = std::move(cb), &cert, nonce = std::move(ocsp_req.second)](
+                        const dht::http::Response& r) {
+                        // Prepare response data
+                        // Verify response validity
+                        if (r.status_code != 200) {
+                            JAMI_WARN("HTTP OCSP Request Failed with code %i", r.status_code);
+                            if (cb)
+                                cb(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
+                            return;
+                        }
+                        JAMI_DBG("HTTP OCSP Request done!");
+                        gnutls_ocsp_cert_status_t verify = GNUTLS_OCSP_CERT_UNKNOWN;
+                        try {
+                            cert.ocspResponse = std::make_shared<dht::crypto::OcspResponse>(
+                                (const uint8_t*) r.body.data(), r.body.size());
+                            JAMI_DBG("%s", cert.ocspResponse->toString().c_str());
+                            verify = cert.ocspResponse->verifyDirect(cert, nonce);
+                        } catch (dht::crypto::CryptoException& e) {
+                            JAMI_ERR("Failed to verify OCSP response: %s", e.what());
+                        }
+                        if (verify == GNUTLS_OCSP_CERT_UNKNOWN) {
+                            // Soft-fail
+                            if (cb)
+                                cb(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
+                            return;
+                        }
+                        int status = GNUTLS_E_SUCCESS;
+                        if (verify == GNUTLS_OCSP_CERT_GOOD) {
+                            JAMI_DBG("OCSP verification success!");
+                        } else {
+                            status = GNUTLS_E_CERTIFICATE_ERROR;
+                            JAMI_ERR("OCSP verification: certificate is revoked!");
+                        }
+                        // Save response into the certificate store
+                        tls::CertificateStore::instance().pinOcspResponse(cert);
+                        if (cb)
+                            cb(status);
+                    });
 }
 
 void
-TlsSession::TlsSessionImpl::sendOcspRequest(const std::string& uri, std::string body,
-                                            std::chrono::seconds timeout, HttpResponse cb)
+TlsSession::TlsSessionImpl::sendOcspRequest(const std::string& uri,
+                                            std::string body,
+                                            std::chrono::seconds timeout,
+                                            HttpResponse cb)
 {
     using namespace dht;
-    auto request = std::make_shared<http::Request>(*Manager::instance().ioContext(), uri);//, logger);
+    auto request = std::make_shared<http::Request>(*Manager::instance().ioContext(),
+                                                   uri); //, logger);
     request->set_method(restinio::http_method_post());
     request->set_header_field(restinio::http_field_t::user_agent, "Jami");
     request->set_header_field(restinio::http_field_t::accept, "*/*");
     request->set_header_field(restinio::http_field_t::content_type, "application/ocsp-request");
     request->set_body(std::move(body));
     request->set_connection_type(restinio::http_connection_header_t::close);
-    request->add_on_state_change_callback([this, cb = std::move(cb), timeout]
-        (const http::Request::State state, const http::Response response)
-    {
-        JAMI_DBG("HTTP OCSP Request state=%i status_code=%i", (unsigned int) state, response.status_code);
-        if (state == http::Request::State::SENDING){
-            auto request = response.request.lock();
-            request->get_connection()->timeout(timeout, [request](const asio::error_code& ec){
-                if (ec and ec != asio::error::operation_aborted)
-                    JAMI_ERR("HTTP OCSP Request timeout with error: %s", ec.message().c_str());
-                request->cancel();
-            });
-        }
-        if (state != http::Request::State::DONE)
-            return;
-        if (cb)
-            cb(response);
-        if (auto request = response.request.lock()) {
-            std::lock_guard<std::mutex> lock(requestsMtx_);
-            requests_.erase(request);
-        }
-    });
+    request->add_on_state_change_callback(
+        [this, cb = std::move(cb), timeout](const http::Request::State state,
+                                            const http::Response response) {
+            JAMI_DBG("HTTP OCSP Request state=%i status_code=%i",
+                     (unsigned int) state,
+                     response.status_code);
+            if (state == http::Request::State::SENDING) {
+                auto request = response.request.lock();
+                request->get_connection()->timeout(timeout, [request](const asio::error_code& ec) {
+                    if (ec and ec != asio::error::operation_aborted)
+                        JAMI_ERR("HTTP OCSP Request timeout with error: %s", ec.message().c_str());
+                    request->cancel();
+                });
+            }
+            if (state != http::Request::State::DONE)
+                return;
+            if (cb)
+                cb(response);
+            if (auto request = response.request.lock()) {
+                std::lock_guard<std::mutex> lock(requestsMtx_);
+                requests_.erase(request);
+            }
+        });
     {
         std::lock_guard<std::mutex> lock(requestsMtx_);
         requests_.emplace(request);
@@ -811,6 +826,7 @@ TlsSession::TlsSessionImpl::sendOcspRequest(const std::string& uri, std::string
 std::size_t
 TlsSession::TlsSessionImpl::send(const ValueType* tx_data, std::size_t tx_size, std::error_code& ec)
 {
+    std::lock_guard<std::mutex> lk(sessionWriteMutex_);
     if (state_ != TlsSessionState::ESTABLISHED) {
         ec = std::error_code(GNUTLS_E_INVALID_SESSION, std::system_category());
         return 0;
@@ -833,7 +849,7 @@ TlsSession::TlsSessionImpl::send(const ValueType* tx_data, std::size_t tx_size,
             nwritten = gnutls_record_send(session_, data_seq, chunck_sz);
         } while ((nwritten == GNUTLS_E_INTERRUPTED and state_ != TlsSessionState::SHUTDOWN)
                  or nwritten == GNUTLS_E_AGAIN);
-        if (nwritten <= 0) {
+        if (nwritten < 0) {
             /* Normally we would have to retry record_send but our internal
              * state has not changed, so we have to ask for more data first.
              * We will just try again later, although this should never happen.
@@ -1018,7 +1034,8 @@ TlsSession::TlsSessionImpl::cleanup()
     stateCondition_.notify_all();
 
     {
-        std::lock_guard<std::mutex> lk(sessionMutex_);
+        std::lock_guard<std::mutex> lk1(sessionReadMutex_);
+        std::lock_guard<std::mutex> lk2(sessionWriteMutex_);
         if (session_) {
             if (transport_->isReliable())
                 gnutls_bye(session_, GNUTLS_SHUT_RDWR);
@@ -1137,7 +1154,8 @@ TlsSession::TlsSessionImpl::handleStateHandshake(TlsSessionState state)
         JAMI_DBG("[TLS] handshake");
         ret = gnutls_handshake(session_);
     } while ((ret == GNUTLS_E_INTERRUPTED or ret == GNUTLS_E_AGAIN)
-             and ++retry_count < HANDSHAKE_MAX_RETRY);
+             and ++retry_count < HANDSHAKE_MAX_RETRY
+             and state_.load() != TlsSessionState::SHUTDOWN);
 
     // Stop on fatal error
     if (gnutls_error_is_fatal(ret)) {
@@ -1603,11 +1621,6 @@ TlsSession::shutdown()
 std::size_t
 TlsSession::write(const ValueType* data, std::size_t size, std::error_code& ec)
 {
-    if (pimpl_->state_ != TlsSessionState::ESTABLISHED) {
-        ec = std::make_error_code(std::errc::broken_pipe);
-        return 0;
-    }
-
     return pimpl_->send(data, size, ec);
 }
 
@@ -1624,7 +1637,7 @@ TlsSession::read(ValueType* data, std::size_t size, std::error_code& ec)
     while (true) {
         ssize_t ret;
         {
-            std::lock_guard<std::mutex> lk(pimpl_->sessionMutex_);
+            std::lock_guard<std::mutex> lk(pimpl_->sessionReadMutex_);
             if (!pimpl_->session_)
                 return 0;
             ret = gnutls_record_recv(pimpl_->session_, data, size);
