@@ -33,6 +33,7 @@
     </message>
     <message>
         <location filename="../src/api/call.h" line="71"/>
+        <location filename="../src/chatview.cpp" line="63"/>
         <source>Connecting</source>
         <translation>Menyambung</translation>
     </message>
@@ -49,6 +50,7 @@
     <message>
         <location filename="../src/api/call.h" line="77"/>
         <location filename="../src/api/call.h" line="83"/>
+        <location filename="../src/chatview.cpp" line="70"/>
         <source>Finished</source>
         <translation type="unfinished"/>
     </message>
@@ -110,8 +112,8 @@
         <translation type="unfinished"/>
     </message>
     <message>
-        <location filename="../src/avmodel.cpp" line="330"/>
-        <location filename="../src/avmodel.cpp" line="349"/>
+        <location filename="../src/avmodel.cpp" line="335"/>
+        <location filename="../src/avmodel.cpp" line="354"/>
         <source>default</source>
         <translation type="unfinished"/>
     </message>
@@ -151,6 +153,7 @@
         <translation type="unfinished"/>
     </message>
     <message>
+        <location filename="../src/chatview.cpp" line="64"/>
         <location filename="../src/newcallmodel.cpp" line="60"/>
         <source>Accepted</source>
         <translation type="unfinished"/>
@@ -403,59 +406,82 @@
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="32"/>
-        <source>Place audio call</source>
+        <source>Show available plugins</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="33"/>
-        <source>Add to conversations</source>
+        <source>Place audio call</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="34"/>
-        <source>Unban contact</source>
+        <source>Add to conversations</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="35"/>
-        <source>Send</source>
+        <source>Unban contact</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="36"/>
-        <source>Options</source>
+        <source>Send</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="37"/>
-        <source>Jump to latest</source>
+        <source>Options</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="38"/>
-        <source>Send file</source>
+        <source>Jump to latest</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="39"/>
-        <source>Leave video message</source>
+        <source>Send file</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="40"/>
-        <source>Leave audio message</source>
+        <source>Add emoji</source>
         <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="41"/>
-        <location filename="../src/chatview.cpp" line="58"/>
-        <source>Accept</source>
-        <translation>Terima</translation>
+        <source>Leave video message</source>
+        <translation type="unfinished"/>
     </message>
     <message>
         <location filename="../src/chatview.cpp" line="42"/>
-        <location filename="../src/chatview.cpp" line="59"/>
-        <source>Refuse</source>
+        <source>Leave audio message</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="44"/>
+        <source>Write to {0}</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="62"/>
+        <source>Unable to make contact</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="67"/>
+        <source>Waiting for contact</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="68"/>
+        <source>Incoming transfer</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="69"/>
+        <source>Timed out waiting for contact</source>
         <translation type="unfinished"/>
     </message>
     <message>
@@ -463,11 +489,6 @@
         <source>Block</source>
         <translation type="unfinished"/>
     </message>
-    <message>
-        <location filename="../src/chatview.cpp" line="44"/>
-        <source>Type a message</source>
-        <translation type="unfinished"/>
-    </message>
     <message>
         <location filename="../src/chatview.cpp" line="46"/>
         <source>Note: an interaction will create a new contact.</source>
@@ -498,6 +519,16 @@
         <source>{0} minutes ago</source>
         <translation type="unfinished"/>
     </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="65"/>
+        <source>Canceled</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="66"/>
+        <source>Ongoing</source>
+        <translation type="unfinished"/>
+    </message>
     <message>
         <location filename="../src/chatview.cpp" line="50"/>
         <source>%d days ago</source>
@@ -533,6 +564,16 @@
         <source>Failure</source>
         <translation type="unfinished"/>
     </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="58"/>
+        <source>Confirm</source>
+        <translation type="unfinished"/>
+    </message>
+    <message>
+        <location filename="../src/chatview.cpp" line="59"/>
+        <source>Deny</source>
+        <translation type="unfinished"/>
+    </message>
     <message>
         <location filename="../src/chatview.cpp" line="60"/>
         <source>Delete</source>
@@ -547,22 +588,22 @@
 <context>
     <name>lrc::ContactModelPimpl</name>
     <message>
-        <location filename="../src/contactmodel.cpp" line="455"/>
+        <location filename="../src/contactmodel.cpp" line="459"/>
         <source>Searching…</source>
         <translation type="unfinished"/>
     </message>
     <message>
-        <location filename="../src/contactmodel.cpp" line="950"/>
+        <location filename="../src/contactmodel.cpp" line="955"/>
         <source>Invalid ID</source>
         <translation type="unfinished"/>
     </message>
     <message>
-        <location filename="../src/contactmodel.cpp" line="953"/>
+        <location filename="../src/contactmodel.cpp" line="958"/>
         <source>Registered name not found</source>
         <translation type="unfinished"/>
     </message>
     <message>
-        <location filename="../src/contactmodel.cpp" line="956"/>
+        <location filename="../src/contactmodel.cpp" line="961"/>
         <source>Couldn&apos;t lookup…</source>
         <translation type="unfinished"/>
     </message>
@@ -570,7 +611,7 @@
 <context>
     <name>lrc::api::ContactModel</name>
     <message>
-        <location filename="../src/contactmodel.cpp" line="419"/>
+        <location filename="../src/contactmodel.cpp" line="420"/>
         <source>Bad URI scheme</source>
         <translation type="unfinished"/>
     </message>
